/** @file wiicom_libcommunication.c
 *  @brief Compact module function library
 *
 * A library that allows you to communicate with the Wiicom Compact modules
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.1.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @attention This library depends on @b wiicom_libutil and on @b wiicom_libsocket \n
 *  Every function that expects a result, in case of an error, set the Wiicom errno and print to stderr the error occurred.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */

#include "wiicom_libcommunication.h"
#include "wiicom_libsocket.h"

#define MAXBUFFER_RECIV_CMD

#define WIICOM_CMD_MSG_HEAD1 'S'
#define WIICOM_CMD_MSG_HEAD2 'T'
#define WIICOM_CMD_MSG_TERM1 'E'
#define WIICOM_CMD_MSG_TERM2 'N'

#define WIICOM_CMD_HEADER_SIZE 9	// [bytes] - byte[0][1]="ST", byte[2][3]= msg_counter, byte[4]=dest, byte[5][6]= data_length, byte[7][8]=action,
#define WIICOM_CMD_TERMINATOR_SIZE 4 // [bytes] - byte[0][1]=CRC, bye[2][3]="EN"

#define WIICOM_DIO_MAX 4

typedef enum
{
    CMD_STEP_RECV_START,
    CMD_STEP_RECV_HEAD1,
    CMD_STEP_RECV_HEAD2,
    CMD_STEP_RECV_MSGCNT1,
    CMD_STEP_RECV_MSGCNT2,
    CMD_STEP_RECV_DEST,
    CMD_STEP_RECV_DATALEN1,
    CMD_STEP_RECV_DATALEN2,
    CMD_STEP_RECV_ACTION1,
    CMD_STEP_RECV_ACTION2,
    CMD_STEP_RECV_DATA,
    CMD_STEP_RECV_CRC1,
    CMD_STEP_RECV_CRC2,
    CMD_STEP_RECV_TERM1,
    CMD_STEP_RECV_TERM2,
    CMD_STEP_RECV_END
} t_wiicom_cmd_msg_step_elab;


const t_dtype_size CMD_DTYPE_SIZE[]= {
	// network configuration data
	{1,1},	//CMD_DTYPE_BAND
	{1,1},	//CMD_DTYPE_POWERMODE
	{6,1},	//CMD_DTYPE_MACADDRESS
	{4,1},	//CMD_DTYPE_SCANCHANNEL
	{32,1},	//CMD_DTYPE_SCANSSID
	{1,1},	//CMD_DTYPE_NWTYPE
	{1,1},	//CMD_DTYPE_SECURITYTYPE
	{1,1},	//CMD_DTYPE_DATARATE
	{1,1},	//CMD_DTYPE_POWERLEVEL
	{63,1},	//CMD_DTYPE_PSK
	{32,1},	//CMD_DTYPE_SSID
	{1,1},	//CMD_DTYPE_IBSSMODE
	{1,1},	//CMD_DTYPE_IBSSCHANNEL
	{1,1},	//CMD_DTYPE_DHCPMODE
	{4,1},	//CMD_DTYPE_IPADDR
	{4,1},	//CMD_DTYPE_NETMASK
	{4,1},	//CMD_DTYPE_GATEWAY
	{4,1},	//CMD_DTYPE_DNSIP
	{42,1},	//CMD_DTYPE_DOMAINNAME
	{2,1},	//CMD_DTYPE_LISTENINTERVAL
	// socket configuration data
	{2,1},	//CMD_DTYPE_SOCKETTYPE
	{2,1},	//CMD_DTYPE_MODULESOCKET
	{2,1},	//CMD_DTYPE_DESTSOCKET
	{4,1},	//CMD_DTYPE_DESTIPADDR
	{1,1},	//CMD_DTYPE_SAVEPARGROUP
	// USDB data
	{1,1},	//CMD_DTYPE_USDB_AI
	{1,1},	//CMD_DTYPE_USDB_AO
	{1,1},	//CMD_DTYPE_USDB_DI
	{1,1},	//CMD_DTYPE_USDB_DO
	{1,1},	//CMD_DTYPE_USDB_BUS
	{1,1},	//CMD_DTYPE_USDB_PERIOD
	{1,1},	//CMD_DTYPE_USDB_TRIGGER
	{1,1},	//CMD_DTYPE_USDB_TIMESTAMP
	{1,1}, //CMD_DTYPE_UART_STOPBITS
	{1,1}, //CMD_DTYPE_UART_PARITY
	{4,1}, //CMD_DTYPE_UART_BAUDRATE
	{1,1}, //CMD_DTYPE_UART_DATABITS
    {18,8}, //CMD_DTYPE_SOCKETCONFIG
    {67,1}, //CMD_DTYPE_INFO_ALL
    {32,1}, //CMD_DTYPE_INFO_ALIAS
    {5,50},   //CMD_DTYPE_CANDB_MSG
    {8,14},   //CMD_DTYPE_CANDB_SIG
    {1,1},   //CMD_DTYPE_CAN_MSG
    {4,14},   //CMD_DTYPE_CAN_SIG
    {11,1}, // CMD_DTYPE_CANPORTCONFIG
    {3,4}, // CMD_DTYPE_DIOCONFIG
    {44,1}, //CMD_DTYPE_ADVANCED
    {5,50},   //CMD_DTYPE_CANDB_MSG_INPUT
    {7,1}, //CMD_DTYPE_I2C
	{1,1}, //CMD_DTYPE_CANBUS_FILTER
	{3,1}, //CMD_DTYPE_USERTASK
	{12,1}, //CMD_DTYPE_POWERSAVE
	{5,1}, //CMD_DTYPE_POWERSAVE_SLEEPNOW
	{4,1} //CMD_DTYPE_POWERSAVE_SLEEPNOW
};

/* =====================================================================================================
    LOCAL FUNCTION/VARIABLE - START
   ===================================================================================================== */

#if (defined (__WIN32__))
static void loc_socket_win_msg(DWORD errCode)
{
    LPSTR errString = NULL;
    int size = FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
							 0,errCode,0,(LPSTR)&errString,0,0 );
    if(size>0)  wiicom_util_set_err_msg(errString);
    else  wiicom_util_set_err_msg("Generic Windows socket error");

    LocalFree( errString ) ;
}
#endif

static int32_t loc_strpos(char *search_str, char *target)
{
   char *p = strstr(target,search_str);
   if(p)    return(p-target);

   return(-1);
}

// used in wiicom_comm_get_canbus_conf and wiicom_comm_get_canbus_conf
static struct
{
    uint32_t speed;           /* CAN speed [bps] */
    uint8_t prseg;           /* CAN PRSEG register  (0-7) */
    uint8_t seg1ph;          /* CAN SEG1PH register (0-7) */
    uint8_t seg2ph;          /* CAN SEG1PH register (0-7) */
    uint8_t sjw;             /* CAN SJW register (0-3) */
    uint8_t sam;             /* CAN SAM register: 0=one_sample, 1=three_samples */
	uint8_t output;
	uint8_t input;
} loc_CanPortConfig;

/* =====================================================================================================
    LOCAL FUNCTION/VARIABLE - END
   ===================================================================================================== */

void wiicom_comm_reset(int8_t b_reset_all,t_wiicom_comm_msg *cmd_msg)
{
    if(b_reset_all==TRUE)
        memset((void *)cmd_msg,'\0',sizeof(t_wiicom_comm_msg));
    else
    {
        memset( (void*)cmd_msg->head,'\0',sizeof(cmd_msg->head));
        memset( (void*)cmd_msg->msgCnt,'\0',sizeof(cmd_msg->msgCnt));
        cmd_msg->dest=0;
        memset( (void*)cmd_msg->dataLength,'\0',sizeof(cmd_msg->dataLength));
        memset( (void*)cmd_msg->action,'\0',sizeof(cmd_msg->action));
        memset( (void*)cmd_msg->data,'\0',sizeof(cmd_msg->data));
        memset( (void*)cmd_msg->crc,'\0',sizeof(cmd_msg->crc));
        memset( (void*)cmd_msg->term,'\0',sizeof(cmd_msg->term));
        cmd_msg->step_elab=CMD_STEP_RECV_START;
	}
}

void  wiicom_comm_print(FILE *fp,t_wiicom_comm_msg *cmd_msg)
{
    int i=0; uint16_t data_length=0;

    data_length=wiicom_util_isBigEndianSystem()?*((uint16_t*)cmd_msg->dataLength):wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength)) ;

    fprintf(fp,"============================WIICOM_COMMAND_MSG=================================\n");
	fprintf(fp,"\t head=[%x][%x]\n",cmd_msg->head[0],cmd_msg->head[1]);
	fprintf(fp,"\t msgCnt=[%x][%x]->[%x]\n",cmd_msg->msgCnt[0],cmd_msg->msgCnt[1],
			wiicom_util_isBigEndianSystem()?*((uint16_t*)cmd_msg->msgCnt):wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->msgCnt) ) );
	fprintf(fp,"\t dest=[%x]\n",cmd_msg->dest);
	fprintf(fp,"\t dataLength=[%x][%x]->[%x]\n",cmd_msg->dataLength[0],cmd_msg->dataLength[1],data_length);
	fprintf(fp,"\t action=[%x][%x]->[%x]\n",cmd_msg->action[0],cmd_msg->action[1],
			wiicom_util_isBigEndianSystem()?*((uint16_t*)cmd_msg->action):wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->action) ) );
	fprintf(fp,"\t crc=[%x][%x]->[%x]\n",cmd_msg->crc[0],cmd_msg->crc[1],
			wiicom_util_isBigEndianSystem()?*((uint16_t*)cmd_msg->crc):wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->crc) ) );
	fprintf(fp,"\t term=[%x][%x]\n\n",cmd_msg->term[0],cmd_msg->term[1]);

    fprintf(fp,"\t data= [");
    for(i=0;i<data_length;i++)   fprintf(fp,"%x ",cmd_msg->data[i]);
    fprintf(fp,"]\n\n");

    fprintf(fp,"\t HEX COMM= [");
    for(i=0;i<WIICOM_CMD_HEADER_SIZE+data_length;i++)   fprintf(fp,"%x ",*((uint8_t *)cmd_msg+i));
    fprintf(fp,"%x %x %x %x",cmd_msg->crc[0],cmd_msg->crc[1],cmd_msg->term[0],cmd_msg->term[1]);
    fprintf(fp,"]\n");


	fprintf(fp,"\t stato_elab=%d idx_buffer=[%d], idx_stop_buffer[%d], idx_data=[%d]\n",cmd_msg->step_elab,cmd_msg->idx_buffer,cmd_msg->idx_stop_buffer,cmd_msg->idx_data);
	fprintf(fp,"===============================================================================\n\n");
}


void wiicom_comm_elab(uint8_t *rxData,uint32_t num_rxData,t_wiicom_comm_msg *cmd_msg, void (*callback)(t_wiicom_comm_msg*), void (*callback_error)(t_wiicom_comm_msg*))
{
    // copio i dati nel buffer solo se ci sta
    if( (uint32_t)(WIICOM_CMD_MAX_BUFFER_DATA-cmd_msg->idx_buffer)>=num_rxData)
    {
        memcpy((void *)(cmd_msg->buffer+cmd_msg->idx_stop_buffer),(void *)rxData,num_rxData);
        cmd_msg->idx_stop_buffer+=num_rxData;
    }

    if(cmd_msg->step_elab==CMD_STEP_RECV_END)   wiicom_comm_reset(FALSE,cmd_msg);

    while(cmd_msg->idx_stop_buffer>0)   //continuo finchè ho dati nel buffer
    {
        uint8_t b_error=FALSE;

        for(; cmd_msg->idx_buffer < cmd_msg->idx_stop_buffer && cmd_msg->step_elab < CMD_STEP_RECV_ACTION2; cmd_msg->idx_buffer++)
        {
            switch(cmd_msg->step_elab)
            {
                case CMD_STEP_RECV_START:
                    if(cmd_msg->buffer[cmd_msg->idx_buffer]==WIICOM_CMD_MSG_HEAD1)
                    {
                        cmd_msg->head[0]=cmd_msg->buffer[cmd_msg->idx_buffer];
                        cmd_msg->step_elab=CMD_STEP_RECV_HEAD1;
                    }
					break;

                case CMD_STEP_RECV_HEAD1:
                    if(cmd_msg->buffer[cmd_msg->idx_buffer]==WIICOM_CMD_MSG_HEAD2)
                    {
                        cmd_msg->head[1]=cmd_msg->buffer[cmd_msg->idx_buffer];
                        cmd_msg->step_elab=CMD_STEP_RECV_HEAD2;
                    }
                    else
                    {
                        // se quello che trovo è 'S' ritorno allo step CMD_STEP_RECV_HEAD1
                        if(cmd_msg->buffer[cmd_msg->idx_buffer]==WIICOM_CMD_MSG_HEAD1)
                        {
                            cmd_msg->head[1]=cmd_msg->buffer[cmd_msg->idx_buffer];
                            cmd_msg->step_elab=CMD_STEP_RECV_HEAD1;
                        }
                        else
                            b_error=TRUE;
                    }
					break;

                case CMD_STEP_RECV_HEAD2:
                    cmd_msg->msgCnt[0]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_MSGCNT1;
					break;

                case CMD_STEP_RECV_MSGCNT1:
                    cmd_msg->msgCnt[1]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_MSGCNT2;
					break;

                case CMD_STEP_RECV_MSGCNT2:
                    cmd_msg->dest=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_DEST;
					break;

                case CMD_STEP_RECV_DEST:
                    cmd_msg->dataLength[0]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_DATALEN1;
					break;

                case CMD_STEP_RECV_DATALEN1:
                    cmd_msg->dataLength[1]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_DATALEN2;
					break;

                case CMD_STEP_RECV_DATALEN2:
                    cmd_msg->action[0]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_ACTION1;
					break;

                case CMD_STEP_RECV_ACTION1:
                    cmd_msg->action[1]=cmd_msg->buffer[cmd_msg->idx_buffer];
                    cmd_msg->step_elab=CMD_STEP_RECV_ACTION2;
                    cmd_msg->idx_data=0;
					break;

				default:
					break;
            }
        }

        // verifico se devo leggere i dati oppure no
        if(b_error==FALSE)
        {
            if(cmd_msg->step_elab==CMD_STEP_RECV_ACTION2)
            {
                uint16_t cmd_data_length=0;
				uint16_t da_leggere=0;
				uint16_t leggibili=0;

                if(wiicom_util_isBigEndianSystem())
                    cmd_data_length=*((uint16_t*)cmd_msg->dataLength);
                else
                    cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );

                da_leggere = cmd_data_length - cmd_msg->idx_data;
                leggibili= cmd_msg->idx_stop_buffer - cmd_msg->idx_buffer;

                if(da_leggere<=leggibili)
                {
                    memcpy( (void *)(cmd_msg->data+cmd_msg->idx_data), (void *)(cmd_msg->buffer+cmd_msg->idx_buffer), da_leggere);
                    cmd_msg->step_elab=CMD_STEP_RECV_DATA;
                    cmd_msg->idx_buffer+=da_leggere;
                    cmd_msg->idx_data+=da_leggere;
                }
                else
                {
                    memcpy( (void *)(cmd_msg->data+cmd_msg->idx_data), (void *)(cmd_msg->buffer+cmd_msg->idx_buffer), leggibili);
                    cmd_msg->idx_buffer+=leggibili;
                    cmd_msg->idx_data+=leggibili;
                }
            }

            // leggo CRC
            if(cmd_msg->step_elab==CMD_STEP_RECV_DATA && cmd_msg->idx_buffer < cmd_msg->idx_stop_buffer)
            {
                cmd_msg->crc[0]=cmd_msg->buffer[cmd_msg->idx_buffer++];
                cmd_msg->step_elab=CMD_STEP_RECV_CRC1;
            }
            if(cmd_msg->step_elab==CMD_STEP_RECV_CRC1 && cmd_msg->idx_buffer < cmd_msg->idx_stop_buffer)
            {
                cmd_msg->crc[1]=cmd_msg->buffer[cmd_msg->idx_buffer++];
                cmd_msg->step_elab=CMD_STEP_RECV_CRC2;
            }

            // leggo terminatore
            if(cmd_msg->step_elab==CMD_STEP_RECV_CRC2  && cmd_msg->idx_buffer < cmd_msg->idx_stop_buffer)
            {
                if(cmd_msg->buffer[cmd_msg->idx_buffer]==WIICOM_CMD_MSG_TERM1)
                {
                    cmd_msg-> step_elab=CMD_STEP_RECV_TERM1;
                    cmd_msg->term[0]=cmd_msg->buffer[cmd_msg->idx_buffer++];
                }
                else
                    b_error=TRUE;
            }
        }

        if(cmd_msg->step_elab==CMD_STEP_RECV_TERM1 && b_error==FALSE && cmd_msg->idx_buffer < cmd_msg->idx_stop_buffer)
        {
            if(cmd_msg->buffer[cmd_msg->idx_buffer]==WIICOM_CMD_MSG_TERM2)
            {
                cmd_msg->step_elab=CMD_STEP_RECV_TERM2;
                cmd_msg->term[1]=cmd_msg->buffer[cmd_msg->idx_buffer++];
            }
            else
                b_error=TRUE;
        }

        //se non ci sono errori verifico il CRC
        if(cmd_msg->step_elab==CMD_STEP_RECV_TERM2 && b_error==FALSE)
        {
            uint16_t crc_cmd=0;
            uint16_t crc=0;
            uint16_t cmd_data_length=0;

            if(wiicom_util_isBigEndianSystem())
                cmd_data_length= *((uint16_t*)cmd_msg->dataLength);
            else
                cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );

            if(wiicom_util_isBigEndianSystem())
                crc_cmd=*((uint16_t*)cmd_msg->crc);
            else
                crc_cmd=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->crc) );

            crc=wiicom_util_CRC16((uint8_t *)cmd_msg,cmd_data_length+WIICOM_CMD_HEADER_SIZE);
            if(crc!=crc_cmd)
                b_error=TRUE;
            else
                cmd_msg->step_elab=CMD_STEP_RECV_END;
        }

        // se sono in presenza di un errore ripulisco la struttura dati che ho caricato
        if(b_error==TRUE)
        {
            wiicom_util_set_errno(WIICOM_EELABCOMM_MSG);

            if(callback_error!=NULL)    callback_error(cmd_msg);
            wiicom_comm_reset(FALSE,cmd_msg);
        }
        else
        {
            if(cmd_msg->step_elab==CMD_STEP_RECV_END)
            {
                if(callback!=NULL)  callback(cmd_msg);
            }
        }

        // svuoto il buffer dei dati che ho elaborato
        memcpy( (void *)cmd_msg->buffer, (void*) (cmd_msg->buffer+cmd_msg->idx_buffer), (cmd_msg->idx_stop_buffer-cmd_msg->idx_buffer));
        cmd_msg->idx_stop_buffer-=cmd_msg->idx_buffer;
        cmd_msg->idx_buffer=0;

        if(cmd_msg->step_elab==CMD_STEP_RECV_END) return;
    }
}

void wiicom_comm_createCommand(t_wiicom_comm_msg *cmd_msg,uint16_t par_msg_cnt,
							   uint8_t par_destination,uint16_t par_action)
{
	wiicom_comm_reset(TRUE,cmd_msg);
	cmd_msg->head[0]=WIICOM_CMD_MSG_HEAD1;
	cmd_msg->head[1]=WIICOM_CMD_MSG_HEAD2;
	cmd_msg->term[0]=WIICOM_CMD_MSG_TERM1;
	cmd_msg->term[1]=WIICOM_CMD_MSG_TERM2;

	cmd_msg->dest=par_destination;

	if(wiicom_util_isBigEndianSystem())
	{
		cmd_msg->msgCnt[0]=par_msg_cnt&0xff;
		cmd_msg->msgCnt[1]=(par_msg_cnt>>8)&0xff;
		cmd_msg->action[0]=par_action&0xff;
		cmd_msg->action[1]=(par_action>>8)&0xff;
	}
	else
	{
		cmd_msg->msgCnt[0]=(par_msg_cnt>>8)&0xff;
		cmd_msg->msgCnt[1]=par_msg_cnt&0xff;
		cmd_msg->action[0]=(par_action>>8)&0xff;
		cmd_msg->action[1]=par_action&0xff;
	}
}

//TODO: manca il controllo sulla lunghezza di par_data
uint8_t wiicom_comm_addToDataBlock(t_wiicom_comm_msg *cmd_msg,uint16_t par_type,
								   uint8_t par_data_cnt,uint8_t par_data_idx,uint8_t *par_data)
{
	uint8_t b_error=FALSE;

	if(par_type>CMD_DTYPE_num_values)	b_error=TRUE;
	if(!b_error && (par_data_cnt<1 || (par_data_cnt+par_data_idx) > CMD_DTYPE_SIZE[par_type].max_data_count) )   	b_error=TRUE;

	if(!b_error)
	{
		cmd_msg->data[cmd_msg->idx_data++]=par_data_cnt;
		if(wiicom_util_isBigEndianSystem())
		{
			cmd_msg->data[cmd_msg->idx_data++]=par_type&0xff;
			cmd_msg->data[cmd_msg->idx_data++]=(par_type>>8)&0xff;
		}
		else
		{
			cmd_msg->data[cmd_msg->idx_data++]=(par_type>>8)&0xff;
			cmd_msg->data[cmd_msg->idx_data++]=par_type&0xff;
		}
        cmd_msg->data[cmd_msg->idx_data++]=par_data_idx;

		if(par_data!=NULL)
		{
			memcpy((void *)(cmd_msg->data+cmd_msg->idx_data), (void *)par_data,par_data_cnt*CMD_DTYPE_SIZE[par_type].bytes_size);
			cmd_msg->idx_data+=(par_data_cnt*CMD_DTYPE_SIZE[par_type].bytes_size);
		}
	}

    if(b_error==TRUE)  wiicom_util_set_errno(WIICOM_EADDDATABLOCK);

	return(!b_error);
}

void wiicom_comm_closeCommand(t_wiicom_comm_msg *cmd_msg)
{
	uint16_t crc=0;

	if(wiicom_util_isBigEndianSystem())
	{
		cmd_msg->dataLength[0]=cmd_msg->idx_data&0xff;
		cmd_msg->dataLength[1]=(cmd_msg->idx_data>>8)&0xff;
	}
	else
	{
		cmd_msg->dataLength[0]=(cmd_msg->idx_data>>8)&0xff;
		cmd_msg->dataLength[1]=cmd_msg->idx_data&0xff;
	}

	crc=wiicom_util_CRC16((uint8_t *)cmd_msg,cmd_msg->idx_data+WIICOM_CMD_HEADER_SIZE);

	if(wiicom_util_isBigEndianSystem())
	{
		cmd_msg->crc[0]=crc&0xff;
		cmd_msg->crc[1]=(crc>>8)&0xff;
	}
	else
	{
		cmd_msg->crc[0]=(crc>>8)&0xff;
		cmd_msg->crc[1]=crc&0xff;
	}

}

//TODO: manca il controllo sulla lunghezza di par_data
uint8_t wiicom_comm_getDataBlock(t_wiicom_comm_msg *cmd_msg,uint16_t par_type,
								 uint8_t par_data_cnt,uint8_t par_data_idx,uint8_t *par_data)
{
	uint8_t b_error=FALSE;

	if(par_type>CMD_DTYPE_num_values)	b_error=TRUE;
	if(!b_error && (par_data_cnt<1 || (par_data_cnt+par_data_idx) > CMD_DTYPE_SIZE[par_type].max_data_count) )   	b_error=TRUE;

	if(!b_error)
	{
		uint16_t cmd_data_length=0, idx_data=0;

		if(wiicom_util_isBigEndianSystem())
			cmd_data_length= *((uint16_t*)cmd_msg->dataLength);
		else
			cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );

		while(idx_data<cmd_data_length)
		{
			uint8_t cur_data_cnt=0;
			uint16_t cur_data_type=0;
			uint8_t cur_data_idx=0;

			cur_data_cnt=cmd_msg->data[idx_data++];
			cur_data_type=(cmd_msg->data[idx_data++]<<8)&0xff;
			cur_data_type|=cmd_msg->data[idx_data++];
			cur_data_idx=cmd_msg->data[idx_data++];

			// verifico se ho trovato il data_type che sto cercando se si copio i dati ed esco
			if(cur_data_type==par_type)
			{
                if( (par_data_idx+par_data_cnt) > cur_data_cnt) return(FALSE);

				idx_data+=par_data_idx*CMD_DTYPE_SIZE[cur_data_type].bytes_size;
				memcpy((void *)(par_data), (void *)(cmd_msg->data+idx_data), par_data_cnt*CMD_DTYPE_SIZE[cur_data_type].bytes_size);
				return(TRUE);
			}

			idx_data+=cur_data_cnt*CMD_DTYPE_SIZE[cur_data_type].bytes_size;
		}

		//se arrivo qua non ho trovato il data_type
		b_error=TRUE;
	}

    if(b_error==TRUE)  wiicom_util_set_errno(WIICOM_EGETDATABLOCK);

	return(!b_error);
}

void wiicom_comm_cmd_msgTOrow(t_wiicom_comm_msg *cmd_msg,uint8_t* rowData,uint16_t *rowData_length)
{
    uint16_t cmd_data_length=0, i=0;

    if(wiicom_util_isBigEndianSystem())
        cmd_data_length= *((uint16_t*)cmd_msg->dataLength);
    else
        cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );

    for(i=0;i<WIICOM_CMD_HEADER_SIZE+cmd_data_length;i++)   rowData[i]=*((uint8_t *)cmd_msg+i);
    for(i=0; i<WIICOM_CMD_TERMINATOR_SIZE; i++)
        rowData[cmd_data_length+WIICOM_CMD_HEADER_SIZE+i]=*((uint8_t *)cmd_msg+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_MAX_BUFFER_DATA+i);

    *rowData_length=cmd_data_length+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_TERMINATOR_SIZE;
}

uint8_t wiicom_comm_recivIScomplete(t_wiicom_comm_msg *cmd_msg)
{
    if(cmd_msg->step_elab==CMD_STEP_RECV_END)   return(TRUE);

    return(FALSE);
}

uint8_t wiicom_comm_sendCommand(t_wiicom_comm_msg *cmd_msg,const char *ip,uint16_t port)
{
	uint8_t b_result=TRUE;
	int i=0;
	int socketfd=0;
	struct sockaddr_in socket_addr;

	//apro il socket
	if((socketfd = wiicom_socket_create(PF_INET, SOCK_STREAM, 0)) == -1)
	{
		b_result=FALSE;
	}
	else
	{
		//mi connetto
		uint16_t cmd_data_length=0;
		uint16_t cmd_action=0;
		uint8_t *buffer=NULL;
		uint16_t buffer_length=0;

		if(wiicom_util_isBigEndianSystem())
		{
			cmd_data_length= *((uint16_t*)cmd_msg->dataLength);
			cmd_action= *((uint16_t*)cmd_msg->action);
		}
		else
		{
			cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );
			cmd_action=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->action) );
		}

		//preparo il buffer contentente il comando da inviare
		buffer_length=cmd_data_length+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_TERMINATOR_SIZE;
		buffer=(uint8_t*) malloc(buffer_length);

		wiicom_socket_set_address(&socket_addr,ip,port);
		if(wiicom_socket_connect(socketfd,&socket_addr)!=-1 && buffer!=NULL)
		{
			for(i=0;i<WIICOM_CMD_HEADER_SIZE+cmd_data_length;i++)   buffer[i]=*((uint8_t *)cmd_msg+i);
			for(i=0; i<WIICOM_CMD_TERMINATOR_SIZE; i++)
				buffer[cmd_data_length+WIICOM_CMD_HEADER_SIZE+i]=*((uint8_t *)cmd_msg+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_MAX_BUFFER_DATA+i);

			//invio il comando
			if(wiicom_socket_sendall(socketfd, buffer,&buffer_length)!=-1)
			{
				//verifico se devo interpretare la risposta
				if(cmd_action!=CMD_ACT_WRITESILENT)
				{
					//adesso interpreto il risultato del comando
					ssize_t numbytes_reciv=0;
					uint8_t reciv_buffer[WIICOM_CMD_MAX_BUFFER_DATA];

					wiicom_comm_reset(TRUE,cmd_msg);
					do
					{
						numbytes_reciv=wiicom_socket_reciv(socketfd,(void *)reciv_buffer,(size_t)WIICOM_CMD_MAX_BUFFER_DATA);
						if(numbytes_reciv>0)
                            wiicom_comm_elab(reciv_buffer,numbytes_reciv,cmd_msg, NULL,NULL);
					}
					while(numbytes_reciv>0 && cmd_msg->step_elab!=CMD_STEP_RECV_END);

                    //wiicom_comm_print(stdout,cmd_msg);
					if(cmd_msg->step_elab!=CMD_STEP_RECV_END)	b_result=FALSE;
				}
			}
			else
				b_result=FALSE;
		}
		else
			b_result=FALSE;

		if(buffer!=NULL)	free((void *)buffer);
	}

	if(socketfd)
    {
        wiicom_socket_close(socketfd);
        // aggiunta per limitare problemi di invio multiplo comandi al Compact
        // il Compact ci mette del tempo a chiudere la connessione e quindi se si cerca di inviare un messaggio dopo l'altro la
        // connessione fallisce
#if (defined (__WIN32__))
		Sleep(1000);
#else
		sleep(1);
#endif
    }

	return(b_result);
}

uint8_t wiicom_comm_sendCommand_socket(t_wiicom_comm_msg *cmd_msg,int socketfd)
{
	uint8_t b_result=TRUE;
	int i=0;

	//apro il socket
	if(socketfd <= 0)
	{
		b_result=FALSE;
	}
	else
	{
		//mi connetto
		uint16_t cmd_data_length=0;
		uint16_t cmd_action=0;
		uint16_t buffer_length=0;
		uint8_t *buffer=NULL;

		if(wiicom_util_isBigEndianSystem())
		{
			cmd_data_length= *((uint16_t*)cmd_msg->dataLength);
			cmd_action= *((uint16_t*)cmd_msg->action);
		}
		else
		{
			cmd_data_length=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->dataLength) );
			cmd_action=wiicom_util_endianU16Conversion( *((uint16_t*)cmd_msg->action) );
		}

        //preparo il buffer contentente il comando da inviare
        buffer_length=cmd_data_length+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_TERMINATOR_SIZE;
		buffer=(uint8_t*) malloc(buffer_length);
		if(buffer!=NULL)
		{
			for(i=0;i<WIICOM_CMD_HEADER_SIZE+cmd_data_length;i++)   buffer[i]=*((uint8_t *)cmd_msg+i);
			for(i=0; i<WIICOM_CMD_TERMINATOR_SIZE; i++)
				buffer[cmd_data_length+WIICOM_CMD_HEADER_SIZE+i]=*((uint8_t *)cmd_msg+WIICOM_CMD_HEADER_SIZE+WIICOM_CMD_MAX_BUFFER_DATA+i);

			//invio il comando
			if(wiicom_socket_sendall(socketfd, buffer,&buffer_length)!=-1)
			{
				//verifico se devo interpretare la risposta
				if(cmd_action!=CMD_ACT_WRITESILENT)
				{
					//adesso interpreto il risultato del comando
					ssize_t numbytes_reciv=0;
					uint8_t reciv_buffer[WIICOM_CMD_MAX_BUFFER_DATA];

					wiicom_comm_reset(TRUE,cmd_msg);
					do
					{
						numbytes_reciv=wiicom_socket_reciv(socketfd,(void *)reciv_buffer,(size_t)WIICOM_CMD_MAX_BUFFER_DATA);
						if(numbytes_reciv>0)
							wiicom_comm_elab(reciv_buffer,numbytes_reciv,cmd_msg, NULL,NULL);
					}
					while(numbytes_reciv>0 && cmd_msg->step_elab!=CMD_STEP_RECV_END);

					//wiicom_comm_print(stdout,cmd_msg);
					if(cmd_msg->step_elab!=CMD_STEP_RECV_END)	b_result=FALSE;
				}
			}
			else
				b_result=FALSE;

			free((void *)buffer);
		}
		else
			b_result=FALSE;
	}

	return(b_result);
}


uint8_t wiicom_comm_get_compact_info(const char *ip,uint16_t port,t_wiicom_comm_compact_info *info_struct)
{
    uint8_t b_error=FALSE;
    t_wiicom_comm_action action;
    t_wiicom_comm_msg cmd_msg;
	uint8_t *tmp_data=NULL;

	tmp_data=(uint8_t*) malloc(CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALL].bytes_size);
	if(tmp_data!=NULL)
	{
		memset((void *)info_struct,0,sizeof(t_wiicom_comm_compact_info));

		action=CMD_ACT_READONESHOT;
		wiicom_comm_createCommand(&cmd_msg,1,0,action);
		if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_INFO_ALL, 1,0,NULL)==FALSE) b_error=TRUE;
		if(!b_error)
		{
			wiicom_comm_closeCommand(&cmd_msg);
			b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
			if(!b_error)
			{
				memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALL].bytes_size);
				b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_INFO_ALL,1,0,tmp_data);
			}
			if(!b_error)
			{
				strncpy(info_struct->version,(char*)tmp_data,32);
				info_struct->version[32]='\0';
				info_struct->type=tmp_data[32];
				strncpy(info_struct->alias,(char*)(tmp_data+33),32);
				info_struct->alias[32]='\0';
				if(wiicom_util_isBigEndianSystem())
					info_struct->internal_buffer_size=*((uint16_t*)(tmp_data+65));
				else
					info_struct->internal_buffer_size=wiicom_util_endianU16Conversion( *((uint16_t*)(tmp_data+65)) );
			}
		}
		free((void*)tmp_data);
	}
	else
		b_error=TRUE;

    return(!b_error);
}

uint8_t wiicom_comm_set_compact_alias(const char *ip,uint16_t port,char *alias)
{
    uint8_t b_error=FALSE,i=0;
	uint8_t *tmp_data=NULL;

    for(i=0;i<strlen(alias);i++)
    {
        if(alias[i]<32)
        {
            wiicom_util_set_errno(WIICOM_ESETCMP_ALIAS);
            b_error=TRUE;
            break;
        }
    }
    if(!b_error)
	{
		tmp_data=(uint8_t *)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALIAS].bytes_size);
		if(tmp_data==NULL)	b_error=TRUE;
	}

    if(!b_error)
    {
        t_wiicom_comm_action action;
        t_wiicom_comm_msg cmd_msg;

        action=CMD_ACT_WRITE;
        wiicom_comm_createCommand(&cmd_msg,1,0,action);
        memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALIAS].bytes_size);
        strncpy((char *)tmp_data,alias,(strlen(alias)<CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALIAS].bytes_size?strlen(alias):CMD_DTYPE_SIZE[CMD_DTYPE_INFO_ALIAS].bytes_size));

        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_INFO_ALIAS, 1,0,tmp_data)==FALSE) b_error=TRUE;
        if(!b_error)
        {
            tmp_data[0]=4;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SAVEPARGROUP, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }
        if(!b_error)
        {
            wiicom_comm_closeCommand(&cmd_msg);
            b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
        }

		free((void *)tmp_data);
    }
    return(!b_error);
}

uint8_t wiicom_comm_get_compact_uart(const char *ip,uint16_t port,t_wiicom_comm_compact_uart *uart_struct)
{
    uint8_t b_error=FALSE;
    t_wiicom_comm_action action;
    t_wiicom_comm_msg cmd_msg;
	uint8_t *tmp_data=NULL;
    t_wiicom_comm_compact_info info_struct;

    // check compact type
    memset((void *)&info_struct,0,sizeof(t_wiicom_comm_compact_info));
    b_error=!wiicom_comm_get_compact_info(ip,port,&info_struct);
    if(!b_error)
	{
	    if(info_struct.type!='C' && info_struct.type!='S' && info_struct.type!='F' && info_struct.type!='G')
	    {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
	    }
	}

    if(!b_error)
	{
        memset((void *)uart_struct,0,sizeof(t_wiicom_comm_compact_uart));
        action=CMD_ACT_READONESHOT;
        wiicom_comm_createCommand(&cmd_msg,1,0,action);
        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_BAUDRATE, 1,0,NULL)==FALSE) b_error=TRUE;
        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_PARITY, 1,0,NULL)==FALSE) b_error=TRUE;
        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_STOPBITS, 1,0,NULL)==FALSE) b_error=TRUE;

        if(!b_error)
        {
            tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_UART_BAUDRATE].bytes_size);
            if(tmp_data==NULL)	b_error=TRUE;
        }

        if(!b_error)
        {
            wiicom_comm_closeCommand(&cmd_msg);
            b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
            if(!b_error)
            {
                memset((void *)tmp_data,0,sizeof(CMD_DTYPE_SIZE[CMD_DTYPE_UART_BAUDRATE].bytes_size));
                b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_UART_BAUDRATE,1,0,tmp_data);
                if(!b_error)
                {
                    if(wiicom_util_isBigEndianSystem())
                        uart_struct->baudrate=*((uint32_t*)(tmp_data));
                    else
                        uart_struct->baudrate=wiicom_util_endianU32Conversion( *((uint32_t*)(tmp_data)) );
                }
            }
            free((void*)tmp_data);
            if(!b_error)
            {
                tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_UART_PARITY].bytes_size);
                if(tmp_data==NULL)	b_error=TRUE;
            }

            if(!b_error)
            {
                memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_UART_PARITY].bytes_size);
                b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_UART_PARITY,1,0,tmp_data);
                if(!b_error)    uart_struct->parity=tmp_data[0];
                free((void*)tmp_data);
            }
            if(!b_error)
            {
                tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_UART_STOPBITS].bytes_size);
                if(tmp_data==NULL)	b_error=TRUE;
            }

            if(!b_error)
            {
                memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_UART_STOPBITS].bytes_size);
                b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_UART_STOPBITS,1,0,tmp_data);
                if(!b_error)    uart_struct->stopbits=tmp_data[0];
                free((void*)tmp_data);
            }
        }
	}

    return(!b_error);
}

uint8_t wiicom_comm_set_compact_uart(const char *ip,uint16_t port,t_wiicom_comm_compact_uart *uart_struct)
{
    uint8_t b_error=FALSE;
    t_wiicom_comm_compact_info info_struct;


    if( (uart_struct->parity!=0 && uart_struct->parity!=1 && uart_struct->parity!=2) ||
	   (uart_struct->stopbits!=0 && uart_struct->stopbits!=1 && uart_struct->stopbits!=2)
	   )
    {
        wiicom_util_set_errno(WIICOM_ESETCMP_UART);
        b_error=TRUE;
    }

    // check compact type
    memset((void *)&info_struct,0,sizeof(t_wiicom_comm_compact_info));
    b_error=!wiicom_comm_get_compact_info(ip,port,&info_struct);
    if(!b_error)
	{
	    if(info_struct.type!='C' && info_struct.type!='S' && info_struct.type!='F' && info_struct.type!='G')
	    {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
	    }
	}

    if(!b_error)
    {
        t_wiicom_comm_action action;
        t_wiicom_comm_msg cmd_msg;
        uint8_t tmp_data[128];

        action=CMD_ACT_WRITE;
        wiicom_comm_createCommand(&cmd_msg,1,0,action);

        memset((void *)tmp_data,0,sizeof(tmp_data));

        if(wiicom_util_isBigEndianSystem())
            memcpy((void*)tmp_data,(void*)&(uart_struct->baudrate),CMD_DTYPE_SIZE[CMD_DTYPE_UART_BAUDRATE].bytes_size);
        else
        {
            tmp_data[3]=uart_struct->baudrate&0xff;
            tmp_data[2]=(uart_struct->baudrate>>8)&0xff;
            tmp_data[1]=(uart_struct->baudrate>>16)&0xff;
            tmp_data[0]=(uart_struct->baudrate>>24)&0xff;
        }
        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_BAUDRATE, 1,0,tmp_data)==FALSE) b_error=TRUE;

        if(!b_error)
        {
            tmp_data[0]=uart_struct->parity;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_PARITY, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            tmp_data[0]=uart_struct->stopbits;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_UART_STOPBITS, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            tmp_data[0]=2;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SAVEPARGROUP, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            wiicom_comm_closeCommand(&cmd_msg);
            b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
        }
    }

    return(!b_error);
}

uint8_t wiicom_comm_get_compact_net(const char *ip,uint16_t port,t_wiicom_comm_compact_net *net_struct)
{
    uint8_t b_error=FALSE;
    t_wiicom_comm_action action;
    t_wiicom_comm_msg cmd_msg;
	uint8_t *tmp_data=NULL;

    memset((void *)net_struct,0,sizeof(t_wiicom_comm_compact_net));

    action=CMD_ACT_READONESHOT;
    wiicom_comm_createCommand(&cmd_msg,1,0,action);

    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_MACADDRESS, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_NWTYPE, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_IPADDR, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_NETMASK, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_GATEWAY, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SSID, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SECURITYTYPE, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_PSK, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_IBSSCHANNEL, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_POWERLEVEL, 1,0,NULL)==FALSE) b_error=TRUE;
    if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_DATARATE, 1,0,NULL)==FALSE) b_error=TRUE;

    if(!b_error)
	{
		tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_MACADDRESS].bytes_size);
		if(tmp_data==NULL)	b_error=TRUE;
	}

    if(!b_error)
    {
        wiicom_comm_closeCommand(&cmd_msg);
        b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_MACADDRESS].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_MACADDRESS,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->macaddress),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_MACADDRESS].bytes_size);
        }
		free((void*)tmp_data);

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_NWTYPE].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_NWTYPE].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_NWTYPE,1,0,tmp_data);
            if(!b_error)
            {
                if(tmp_data[0]==1)  net_struct->nwtype=tmp_data[0];
                else    net_struct->nwtype=0;
            }
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_IPADDR].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
		if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_IPADDR].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_IPADDR,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->ipaddress),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_IPADDR].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_NETMASK].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_NETMASK].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_NETMASK,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->netmask),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_NETMASK].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_GATEWAY].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
		if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_GATEWAY].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_GATEWAY,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->gateway),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_GATEWAY].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_SSID].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
		if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_SSID].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_SSID,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->ssid),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_SSID].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_SECURITYTYPE].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_SECURITYTYPE].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_SECURITYTYPE,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->security_type),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_SECURITYTYPE].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_PSK].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_PSK].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_PSK,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->psk),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_PSK].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_IBSSCHANNEL].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_IBSSCHANNEL].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_IBSSCHANNEL,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->channel),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_IBSSCHANNEL].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_POWERLEVEL].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_POWERLEVEL].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_POWERLEVEL,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->rf_power_level),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_POWERLEVEL].bytes_size);
			free((void*)tmp_data);
        }

		if(!b_error)
		{
			tmp_data=(uint8_t*)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_DATARATE].bytes_size);
			if(tmp_data==NULL)	b_error=TRUE;
		}
        if(!b_error)
        {
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_DATARATE].bytes_size);
            b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_DATARATE,1,0,tmp_data);
            if(!b_error)
                memcpy((void *)&(net_struct->data_rate),(void*)tmp_data,CMD_DTYPE_SIZE[CMD_DTYPE_DATARATE].bytes_size);
			free((void*)tmp_data);
        }
    }

    return(!b_error);
}

uint8_t wiicom_comm_set_compact_net(const char *ip,uint16_t port,t_wiicom_comm_compact_net *net_struct)
{
    uint8_t b_error=FALSE,i=0;

    if( (net_struct->nwtype!=0 && net_struct->nwtype!=1) ||
	   (net_struct->security_type!=0 && net_struct->security_type!=1 && net_struct->security_type!=2 && net_struct->security_type!=3)
	   )
    {
        wiicom_util_set_errno(WIICOM_ESETCMP_NET);
        b_error=TRUE;
    }

    if(!b_error)
    {
        for(i=0;i<strlen(net_struct->psk);i++)
        {
            if(net_struct->psk[i]<32)
            {
                wiicom_util_set_errno(WIICOM_ESETCMP_NET);
                b_error=TRUE;
                break;
            }
        }
    }

    if(!b_error)
    {
        for(i=0;i<strlen(net_struct->ssid);i++)
        {
            if(net_struct->ssid[i]<32)
            {
                wiicom_util_set_errno(WIICOM_ESETCMP_NET);
                b_error=TRUE;
                break;
            }
        }
    }

    if(!b_error)
    {
        t_wiicom_comm_action action;
        t_wiicom_comm_msg cmd_msg;
        uint8_t tmp_data[128];

        action=CMD_ACT_WRITE;
        wiicom_comm_createCommand(&cmd_msg,1,0,action);

        memset((void *)tmp_data,0,sizeof(tmp_data));
        if(net_struct->nwtype==0 && net_struct->security_type!=0)   tmp_data[0]=2;
        else   tmp_data[0]=net_struct->nwtype;
        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_NWTYPE, 1,0,tmp_data)==FALSE) b_error=TRUE;

        if(!b_error)
        {
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_IPADDR, 1,0,(uint8_t*)&(net_struct->ipaddress))==FALSE) b_error=TRUE;
        }
        if(!b_error)
        {
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_NETMASK, 1,0,(uint8_t*)&(net_struct->netmask))==FALSE) b_error=TRUE;
        }
        if(!b_error)
        {
            if(net_struct->nwtype==1)
            {
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_GATEWAY, 1,0,(uint8_t*)&(net_struct->gateway))==FALSE) b_error=TRUE;
            }
            else
            {
                memset((void *)tmp_data,0,sizeof(tmp_data));
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_GATEWAY, 1,0,tmp_data)==FALSE) b_error=TRUE;
            }
        }
        if(!b_error)
        {
            memset((void *)tmp_data,0,sizeof(tmp_data));
            strncpy((char *)tmp_data,net_struct->ssid,(strlen(net_struct->ssid)<CMD_DTYPE_SIZE[CMD_DTYPE_SSID].bytes_size?strlen(net_struct->ssid):CMD_DTYPE_SIZE[CMD_DTYPE_SSID].bytes_size));

            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SSID, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }
        if(!b_error)
        {
            memset((void *)tmp_data,0,sizeof(tmp_data));
            if(net_struct->security_type!=0)
                strncpy((char *)tmp_data,net_struct->psk,(strlen(net_struct->psk)<CMD_DTYPE_SIZE[CMD_DTYPE_PSK].bytes_size?strlen(net_struct->psk):CMD_DTYPE_SIZE[CMD_DTYPE_PSK].bytes_size));

            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_PSK, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SECURITYTYPE, 1,0,&(net_struct->security_type))==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            if(net_struct->channel<1 || net_struct->channel>14) b_error=TRUE;
            else
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_IBSSCHANNEL, 1,0,&(net_struct->channel))==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            if(net_struct->rf_power_level<RF_POWER_LOW || net_struct->rf_power_level>RF_POWER_HIGH) b_error=TRUE;
            else
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_POWERLEVEL, 1,0,(uint8_t *)&(net_struct->rf_power_level))==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            if(net_struct->data_rate<DATA_RATE_AUTO || net_struct->data_rate>DATA_RATE_12Mbs) b_error=TRUE;
            else
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_DATARATE, 1,0,(uint8_t *)&(net_struct->data_rate))==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            tmp_data[0]=1;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SAVEPARGROUP, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }

        if(!b_error)
        {
            wiicom_comm_closeCommand(&cmd_msg);
            b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
        }
    }

    return(!b_error);
}

uint8_t wiicom_comm_get_compact_dio(const char *ip,uint16_t port,uint8_t num_dio,uint8_t *b_enabled,uint8_t *type)
{
    uint8_t b_error=FALSE;
    t_wiicom_comm_action action;
    t_wiicom_comm_msg cmd_msg;
	uint8_t *tmp_data=NULL;
    t_wiicom_comm_compact_info info_struct;

    // check compact type
    memset((void *)&info_struct,0,sizeof(t_wiicom_comm_compact_info));
    b_error=!wiicom_comm_get_compact_info(ip,port,&info_struct);
    if(!b_error)
	{
	    if(info_struct.type!='D' && info_struct.type!='E' && info_struct.type!='S' && info_struct.type!='C' && info_struct.type!='A' && info_struct.type!='B')
	    {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
	    }

        // se CANBUS non posso leggere lo stato dei dio 2 e 3
        if( !b_error && (info_struct.type=='A' || info_struct.type=='B') && (num_dio==2 || num_dio==3) )
        {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
        }

        // se UART non posso leggere lo stato dei dio 0 e 1
        if( !b_error && (info_struct.type=='S' || info_struct.type=='C') && (num_dio==0 || num_dio==1) )
        {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
        }
        if(!b_error && num_dio>=WIICOM_DIO_MAX)  b_error=TRUE;

        if(!b_error)
        {
            tmp_data=(uint8_t*) malloc(CMD_DTYPE_SIZE[CMD_DTYPE_DIOCONFIG].bytes_size);
            if(tmp_data!=NULL)
            {
                action=CMD_ACT_READONESHOT;
                wiicom_comm_createCommand(&cmd_msg,1,0,action);
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_DIOCONFIG, 1,num_dio,NULL)==FALSE) b_error=TRUE;
                if(!b_error)
                {
                    wiicom_comm_closeCommand(&cmd_msg);
                    b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
                    if(!b_error)
                    {
                        memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_DIOCONFIG].bytes_size);
                        b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_DIOCONFIG,1,0,tmp_data);
                    }
                    if(!b_error)
                    {
                        if(*tmp_data==num_dio)
                        {
                            *b_enabled=*(tmp_data+1);
                            *type=*(tmp_data+1);
                        }
                        else
                            b_error=TRUE;
                    }
                }
                free((void*)tmp_data);
            }
            else
                b_error=TRUE;
        }
	}

    return(!b_error);
}

uint8_t wiicom_comm_set_compact_dio(const char *ip,uint16_t port,uint8_t num_dio,uint8_t b_enabled,uint8_t type)
{
    uint8_t b_error=FALSE;
	uint8_t *tmp_data=NULL;
    t_wiicom_comm_compact_info info_struct;

    // check compact type
    memset((void *)&info_struct,0,sizeof(t_wiicom_comm_compact_info));
    b_error=!wiicom_comm_get_compact_info(ip,port,&info_struct);
    if(!b_error)
	{
	    if(info_struct.type!='D' && info_struct.type!='E' && info_struct.type!='S' && info_struct.type!='C' && info_struct.type!='A' && info_struct.type!='B')
	    {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
	    }

        // se CANBUS non posso settare lo stato dei dio 2 e 3
        if( !b_error && (info_struct.type=='A' || info_struct.type=='B') && (num_dio==2 || num_dio==3) )
        {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
        }

        // se UART non posso settare lo stato dei dio 0 e 1
        if( !b_error && (info_struct.type=='S' || info_struct.type=='C') && (num_dio==0 || num_dio==1) )
        {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
        }

        if(!b_error)
        {
            if(num_dio>=WIICOM_DIO_MAX || b_enabled>1 || type>1)  b_error=TRUE;
        }

        if(!b_error)
        {
            tmp_data=(uint8_t *)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_DIOCONFIG].bytes_size);
            if(tmp_data==NULL)	b_error=TRUE;
        }

        if(!b_error)
        {
            t_wiicom_comm_action action;
            t_wiicom_comm_msg cmd_msg;

            action=CMD_ACT_WRITE;
            wiicom_comm_createCommand(&cmd_msg,1,0,action);
            memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_DIOCONFIG].bytes_size);

            tmp_data[0]=num_dio;
            tmp_data[1]=b_enabled;
            tmp_data[2]=type;

            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_DIOCONFIG, 1,num_dio,tmp_data)==FALSE) b_error=TRUE;
            if(!b_error)
            {
                tmp_data[0]=7;
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SAVEPARGROUP, 1,0,tmp_data)==FALSE) b_error=TRUE;
            }
            if(!b_error)
            {
                wiicom_comm_closeCommand(&cmd_msg);
                b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
            }

            free((void *)tmp_data);

        }
	}

    return(!b_error);
}

uint8_t wiicom_comm_get_compact_canbus(const char *ip,uint16_t port,uint32_t *speed,uint8_t *output_type,uint8_t *input_type)
{
    uint8_t b_error=FALSE;
	uint8_t *tmp_data=NULL;
    t_wiicom_comm_compact_info info_struct;

    // check compact type
    memset((void *)&info_struct,0,sizeof(t_wiicom_comm_compact_info));
    b_error=!wiicom_comm_get_compact_info(ip,port,&info_struct);
    if(!b_error)
	{
	    if(info_struct.type!='A' && info_struct.type!='B')
	    {
            wiicom_util_set_errno(WIICOM_ENOTSUPPORTED);
	        b_error=TRUE;
	    }

        if(!b_error)
        {
            tmp_data=(uint8_t*) malloc(CMD_DTYPE_SIZE[CMD_DTYPE_CANPORTCONFIG].bytes_size);
            if(tmp_data!=NULL)
            {
                t_wiicom_comm_action action;
                t_wiicom_comm_msg cmd_msg;

                action=CMD_ACT_READONESHOT;
                wiicom_comm_createCommand(&cmd_msg,1,0,action);
                if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_CANPORTCONFIG, 1,0,NULL)==FALSE) b_error=TRUE;
                if(!b_error)
                {
                    wiicom_comm_closeCommand(&cmd_msg);
                    b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
                    if(!b_error)
                    {
                        memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_CANPORTCONFIG].bytes_size);
                        b_error=!wiicom_comm_getDataBlock(&cmd_msg,CMD_DTYPE_CANPORTCONFIG,1,0,tmp_data);
                    }
                    if(!b_error)
                    {
                        memset((void *)&loc_CanPortConfig,0,sizeof(loc_CanPortConfig));
                        memcpy((void *)&loc_CanPortConfig,(void *)tmp_data,sizeof(loc_CanPortConfig));

                        if(wiicom_util_isBigEndianSystem())
                            loc_CanPortConfig.speed=*((uint32_t*)(tmp_data));
                        else
                            loc_CanPortConfig.speed=wiicom_util_endianU32Conversion( *((uint32_t*)(tmp_data)) );

                        *speed=loc_CanPortConfig.speed;
                        *output_type=loc_CanPortConfig.output;
                        *input_type=loc_CanPortConfig.input;
                    }
                }
                free((void*)tmp_data);
            }
            else
                b_error=TRUE;
        }
	}

    return(!b_error);
}

uint8_t wiicom_comm_set_compact_canbus(const char *ip,uint16_t port,uint32_t speed,uint8_t output_type,uint8_t input_type)
{
    uint8_t b_error=FALSE;
	uint8_t *tmp_data=NULL;
    uint32_t tmp_speed;
    uint8_t tmp_output_type,tmp_input_type;

    b_error=!wiicom_comm_get_compact_canbus(ip,port,&tmp_speed,&tmp_output_type,&tmp_input_type);

    if(!b_error)
    {
        if(output_type>8 || output_type==6 || output_type==7 || input_type>2)    b_error=TRUE;
    }

    if(!b_error)
    {
        tmp_data=(uint8_t *)malloc(CMD_DTYPE_SIZE[CMD_DTYPE_CANPORTCONFIG].bytes_size);
        if(tmp_data==NULL)	b_error=TRUE;
    }

    if(!b_error)
    {
        t_wiicom_comm_action action;
        t_wiicom_comm_msg cmd_msg;

        action=CMD_ACT_WRITE;
        wiicom_comm_createCommand(&cmd_msg,1,0,action);
        memset((void *)tmp_data,0,CMD_DTYPE_SIZE[CMD_DTYPE_CANPORTCONFIG].bytes_size);

        if(wiicom_util_isBigEndianSystem())
            memcpy((void*)tmp_data,(void*)&(speed),4);
        else
        {
            tmp_data[3]=speed&0xff;
            tmp_data[2]=(speed>>8)&0xff;
            tmp_data[1]=(speed>>16)&0xff;
            tmp_data[0]=(speed>>24)&0xff;
        }


        tmp_data[4]=loc_CanPortConfig.prseg;
        tmp_data[5]=loc_CanPortConfig.seg1ph;
        tmp_data[6]=loc_CanPortConfig.seg2ph;
        tmp_data[7]=loc_CanPortConfig.sjw;
        tmp_data[8]=loc_CanPortConfig.sam;
        tmp_data[9]=output_type;
        tmp_data[10]=input_type;

        if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_CANPORTCONFIG, 1,0,tmp_data)==FALSE) b_error=TRUE;
        if(!b_error)
        {
            tmp_data[0]=6;
            if(wiicom_comm_addToDataBlock(&cmd_msg, CMD_DTYPE_SAVEPARGROUP, 1,0,tmp_data)==FALSE) b_error=TRUE;
        }
        if(!b_error)
        {
            wiicom_comm_closeCommand(&cmd_msg);
            b_error=!wiicom_comm_sendCommand(&cmd_msg,ip,port);
        }

        free((void *)tmp_data);

    }

    return(!b_error);
}
uint8_t wiicom_comm_sendMsgCanbus(const char *ip,uint16_t port,uint32_t idmsg,uint8_t b_extendedMsg,uint8_t dlc, uint8_t *data)
{
	uint8_t b_result=TRUE;
	int socketfd=0;
	struct sockaddr_in socket_addr;

    if(dlc==0 || dlc>8 || b_extendedMsg>1)  return(FALSE);

	//apro il socket
	if((socketfd = wiicom_socket_create(PF_INET, SOCK_STREAM, 0)) == -1)    b_result=FALSE;
	else
	{
		char *buffer=NULL;
		uint16_t buffer_length=0;

		//preparo il buffer contentente il messaggio CAN da inviare
		buffer_length=60+dlc*4;
		buffer=(char*) malloc(buffer_length);

        if(buffer==NULL)    b_result=FALSE;
        else
		{
            wiicom_socket_set_address(&socket_addr,ip,port);
            if(wiicom_socket_connect(socketfd,&socket_addr)!=-1)
            {
                uint8_t i=0;
                char str_tmp[10]="";

                memset((void*)buffer,'\0',buffer_length);
                sprintf(buffer, "*,%x,%.2x,%.2x", idmsg, b_extendedMsg, dlc);

                for(i=0;i<dlc;i++)
                {
                    sprintf(str_tmp,",%.2x",data[i]);
                    strcat(buffer,str_tmp);
                }
                strcat(buffer,",#");

                buffer_length=strlen(buffer);
                if(wiicom_socket_sendall(socketfd,(uint8_t*)buffer,&buffer_length)==-1)  b_result=FALSE;
            }
            else
                b_result=FALSE;

            free((void *)buffer);
		}
	}

	if(socketfd)
    {
        wiicom_socket_close(socketfd);
        // aggiunta per limitare problemi di invio multiplo comandi al Compact
        // il Compact ci mette del tempo a chiudere la connessione e quindi se si cerca di inviare un messaggio dopo l'altro la
        // connessione fallisce
#if (defined (__WIN32__))
		Sleep(1000);
#else
		sleep(1);
#endif
    }

	return(b_result);
}



uint8_t wiicom_comm_sendMsgCanbus_socket(int socketfd,uint32_t idmsg,uint8_t b_extendedMsg,uint8_t dlc, uint8_t *data)
{
	uint8_t b_result=TRUE;

    if(dlc==0 || dlc>8 || b_extendedMsg>1)  b_result=FALSE;
    else
	{
		char *buffer=NULL;
		uint16_t buffer_length=0;

		//preparo il buffer contentente il messaggio CAN da inviare
		buffer_length=60+dlc*4;
		buffer=(char*) malloc(buffer_length);

        if(buffer==NULL)    b_result=FALSE;
        else
		{
            uint8_t i=0;
            char str_tmp[10]="";

            memset((void*)buffer,'\0',buffer_length);
            sprintf(buffer, "*,%x,%.2x,%.2x", idmsg, b_extendedMsg, dlc);

            for(i=0;i<dlc;i++)
            {
                sprintf(str_tmp,",%.2x",data[i]);
                strcat(buffer,str_tmp);
            }
            strcat(buffer,",#");

            buffer_length=strlen(buffer);
            if(wiicom_socket_sendall(socketfd,(uint8_t*)buffer,&buffer_length)==-1)  b_result=FALSE;

            free((void *)buffer);
		}
	}

	return(b_result);
}

uint8_t wiicom_comm_parseMsgCanbus(const char *recivData,uint16_t recivedDataLength,uint32_t *idmsg,uint8_t *b_extendedMsg,uint8_t *dlc, uint8_t *data)
{
	uint8_t b_result=TRUE;
    char *buffer=NULL;

	buffer=(char*) malloc(recivedDataLength+1);
    if(buffer!=NULL)
    {
        char *p_buffer=NULL, *pch=NULL, *pEnd=NULL;
        int32_t idx_start=0, idx_end=0;
        uint32_t tmp_idmsg=0,tmp_b_extendedMsg=0,tmp_dlc=0,count_data=0,i=0;
        uint8_t *tmp_data;

        memcpy((void *)buffer,(void *)recivData,recivedDataLength);
        buffer[recivedDataLength]='\0';

        //seach idx_start
        idx_start=loc_strpos("*,",buffer);
        if(idx_start==-1)
        {
            free(buffer);
            return(FALSE);
        }
        p_buffer=buffer+idx_start+2;

        //seach idx_end
        idx_end=loc_strpos(",#",p_buffer);
        if(idx_end==-1)
        {
            free(buffer);
            return(FALSE);
        }
        p_buffer[idx_end]='\0';
        //now p_buffer contain a potential CANBUS message

        //get idmsg
        pch=strtok(p_buffer,",");
        if(pch==NULL)
        {
            free(buffer);
            return(FALSE);
        }
        tmp_idmsg=strtol(pch, &pEnd, 16);

        //get b_extendedMsg
        pch=strtok(NULL,",");
        if(pch==NULL || strlen(pch)>2)
        {
            free(buffer);
            return(FALSE);
        }
        tmp_b_extendedMsg=atoi(pch);
        if(tmp_b_extendedMsg>1)
        {
            free(buffer);
            return(FALSE);
        }

        // test idmsg value
		if( (tmp_b_extendedMsg==0 && tmp_idmsg>0x7FF) || (tmp_b_extendedMsg==1 && tmp_idmsg>0x1FFFFFFF) )
        {
            free(buffer);
            return(FALSE);
        }

        // get dlc
        pch=strtok(NULL,",");
        if(pch==NULL || strlen(pch)>2)
        {
            free(buffer);
            return(FALSE);
        }
        tmp_dlc=atol(pch);
        if(tmp_dlc>8)
        {
            free(buffer);
            return(FALSE);
        }

        // now get CANBUS data
        tmp_data=(uint8_t*) malloc(tmp_dlc);
        for(pch=strtok(NULL,","); pch!=NULL && b_result==TRUE; pch=strtok(NULL,","))
        {
            if(count_data>=tmp_dlc)   b_result=FALSE;
            else
            {
                if(strlen(pch)>2)   b_result=FALSE;
                else
                {
                    //check value
                    for(i=0;i<strlen(pch) && b_result==TRUE;i++)
                    {
                        if( !((pch[i]>0x29 && pch[i]<0x40) || (pch[i]>0x40 && pch[i]<0x47) || (pch[i]>0x60 && pch[i]<0x67)) )   b_result=FALSE;
                    }
                    if(b_result)
                    {

                        tmp_data[count_data]=(uint8_t)strtol(pch, &pEnd, 16);
                        count_data++;
                    }
                }
            }
        }
        if(b_result)
        {
            *idmsg=tmp_idmsg;
            *b_extendedMsg=tmp_b_extendedMsg;
            *dlc=tmp_dlc;
            memcpy((void *)data,(void *)tmp_data,sizeof(tmp_dlc));
        }
        free(tmp_data);
        free(buffer);
    }
    else    b_result=FALSE;

    return(b_result);
}
