/** @file wiicom_libcommunication.h
 *  @brief Compact module function library
 *
 * A library that allows you to communicate with the Wiicom Compact modules
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.1.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @attention This library depends on @b wiicom_libutil and on @b wiicom_libsocket \n
 *  Every function that expects a result, in case of an error, set the Wiicom errno and print to stderr the error occurred.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */

#ifndef _WIICOM_LIBCOMMUNICATINO_H
#define _WIICOM_LIBCOMMUNICATINO_H

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "wiicom_libutil.h"

#define WIICOM_CMD_MAX_BUFFER_DATA	65536	/**< Maximum length of data you can send or receive from the Wiicom Compact modules */

/**
 *  @brief Types of actions that can be requested by a command to the Compact module
 */
typedef enum
{
    CMD_ACT_READONESHOT,	/**< Request a read data action to the Compact module */
    CMD_ACT_WRITE,			/**< Request a write data action to the Compact module */
    CMD_ACT_READPERIODIC,	/**< Request a periodic read data action to the Compact module */
    CMD_ACT_READTRIGGERED,	/**< Request a read data action to the Compact module when an event is trigger */
    CMD_ACT_WRITESILENT,	/**< Request a write data action to the Compact module without echo response */
	CMD_ACT_num_values		/**< Total number of actions available */
}  t_wiicom_comm_action;

/**
 *  @brief Types of data that can be read or can be sent to your Compact
 */
typedef enum
{
    // network configuration data
    CMD_DTYPE_BAND,          	/**< Band (1 bytes) */
	CMD_DTYPE_POWERMODE,     	/**< PowerMode (1 bytes) 0=Power Saving Disabled, 1=RF+Baseband+Control disabled, 2=RF+Baseband disabled) */
    CMD_DTYPE_MACADDRESS,    	/**< Mac address (6 bytes)(MSBfirst)  */
	CMD_DTYPE_SCANCHANNEL,  	/**< Channel (4 bytes)(LSB first) */
	CMD_DTYPE_SCANSSID,      	/**< Ssid (32 bytes)(string) */
	CMD_DTYPE_NWTYPE,        	/**< Network type (1 bytes) 1=INFRASTRUCTURE_MODE, 0=OPEN_MODE (ad hoc), 2=SECURITY_MODE (ad hoc) */
	CMD_DTYPE_SECURITYTYPE,  	/**< Security type (1 bytes) 0=OPEN, 1=WPA1, 2=SECURITY_WPA2, 3=SECURITY_WEP */
	CMD_DTYPE_DATARATE,      	/**< Data rate (1 bytes) 0=DATA_RATE_AUTO,1, 2,5, 11, 6, 9, 12 */
	CMD_DTYPE_POWERLEVEL,    	/**< Power level (1 bytes) 0=POWER_LEVEL_LOW, 1=POWER_LEVEL_MEDIUM, 2=POWER_LEVEL_HIGH */
	CMD_DTYPE_PSK,           	/**< Password (64 bytes)(string) */
	CMD_DTYPE_SSID,          	/**< Ssid (32 bytes) (string) */
	CMD_DTYPE_IBSSMODE,      	/**< Ibssmode (1 bytes) 1=IBSS_CREATOR (ad hoc),  0=IBSS_JOINER (infrastructure) */
	CMD_DTYPE_IBSSCHANNEL,   	/**< Ibss channel (1 bytes) if IBSSMODE creator then set channel: 2.5GHz channels are 1-14 */
	CMD_DTYPE_DHCPMODE,      	/**< Dhcp mode (1 bytes) 0=DHCP_MODE_DISABLE, 1=DHCP_MODE_ENABLE */
	CMD_DTYPE_IPADDR,        	/**< Ip addrress (4 bytes)(MSBfirst) */
    CMD_DTYPE_NETMASK,       	/**< Netmask (4 bytes)(MSBfirst) */
	CMD_DTYPE_GATEWAY,       	/**< Ip gateway (4 bytes)(MSBfirst) */
    CMD_DTYPE_DNSIP,         	/**< Ip dns (4 bytes)(MSBfirst) */
	CMD_DTYPE_DOMAINNAME,    	/**< Domain name (42 bytes)(string) */
	CMD_DTYPE_LISTENINTERVAL, 	/**< Listen interval (2 bytes)(LSBfirst) frequency of module wake-up from AP (to set when in power save) [unit: AP beacons] */

    // socket configuration data
	CMD_DTYPE_SOCKETTYPE,    	/**< Not implemented */
	CMD_DTYPE_MODULESOCKET,  	/**< Not implemented */
	CMD_DTYPE_DESTSOCKET,    	/**< Not implemented */
    CMD_DTYPE_DESTIPADDR,    	/**< Not implemented */
	CMD_DTYPE_SAVEPARGROUP,    	/**< 0=none, 1=save network par in ROM, 2= save UART par in ROM, 3= save socket par in ROM \n
                                4= save alias name, 5=save CANBUS DB par in ROM, 6=save CANBUS PORT par in ROM, 7= save DIO par in ROM \n
                                8= save advance par in ROM (automatically reset to 0 after saving) */

    // USDB data
	CMD_DTYPE_USDB_AI,			/**< Not implemented */
	CMD_DTYPE_USDB_AO,			/**< Not implemented */
	CMD_DTYPE_USDB_DI,			/**< Not implemented */
	CMD_DTYPE_USDB_DO,			/**< Not implemented */
	CMD_DTYPE_USDB_BUS,			/**< Not implemented */
	CMD_DTYPE_USDB_PERIOD,		/**< Not implemented */
	CMD_DTYPE_USDB_TRIGGER,		/**< Not implemented */
	CMD_DTYPE_USDB_TIMESTAMP,	/**< Not implemented */

	// UART config
	CMD_DTYPE_UART_STOPBITS,    /**< UART stop bit (1 bytes) */
	CMD_DTYPE_UART_PARITY,      /**< UART parity (1 bytes) */
	CMD_DTYPE_UART_BAUDRATE,    /**< UART baud rate (4 bytes) */
	CMD_DTYPE_UART_DATABITS,    /**< 1 byte - reserved */

    // socket configuration data
    CMD_DTYPE_SOCKETCONFIG,     /**< 18 byte array  (byte order MSB first): \n
                                 MAXSOCKETS=8 \n
                                 byte[0]                 = socketIdx;            socket idx [0-MAXSOCKETS] \n
                                 byte[1]                 = socketEnable;         0=disabled, 1=enabled \n
                                 byte[2][3]              = socketType;           0=TCP Client, 1=UDP, 2=TCP Server (Listening TCP) \n
                                 byte[4][5]              = localPort;            client/server sockets \n
                                 byte[6][7]              = destPort;             only client sockets \n
                                 byte[8][9][10][11]      = destIP;               only client sockets \n
                                 byte[12]                = tcpConnCloseRule;     0=never, 1=data_tx_timeout, 2=data_rx_timeout, 3= data_tx/rx_timeout \n
                                 byte[13][14][15][16]    = tcpConnCloseTimeout;  0=none, 1 - 4294967296  = timeout [ms] \n
                                 byte[17]                = socketMult;           number of multiple sockets (server only): 1 = single socket, [2-MAXSOCKETS]=multiple socket
                                 */
    CMD_DTYPE_INFO_ALL, /**< 67 byte array  (byte order MSB first): \n
                         byte[0][31]                 = swVersion firmware version (string) \n
                         byte[32]                    = compactType 'S' = server, 'C'=client/server \n
                         byte[33][64]                = compactAlias (custom defined name) (string) \n
                         byte[65][66]                = usdbWifiCmd max size [bytes] \n
                         */
    CMD_DTYPE_INFO_ALIAS, /**< 32 byte array (MSBfirst) \n
                           compact alias (custom defined name) (string)
                           */
	CMD_DTYPE_CANDB_MSG, /** 5 byte array  (byte order MSB first):
	                             byte[0][3]  msgId;          = CAN message - msg ID
	                             byte[4] msgIdExt;           = CAN message - msg IDtype (extended,standard)
                         */
	CMD_DTYPE_CANDB_SIG, /** 8 byte array  (byte order MSB first):
	                             byte[0][3]  msgId;          = CAN signal - msg ID
	                             byte[4] msgIdExt;           = CAN signal - msg IDtype (extended,standard))
	                             byte[5]   startBit;         = CAN signal - startbit (0-based)
	                             byte[6]   length;           = CAN signal - length [bit]
	                             byte[7]   byteOrder;        = CAN signal - byte order (MOTOROLA=1,INTEL=0)
	                         */
	CMD_DTYPE_CAN_MSG,  /** Reserved **/
	CMD_DTYPE_CAN_SIG,  /**  4 byte array (MSBfirst)    = CAN signal - raw value **/
	CMD_DTYPE_CANPORTCONFIG, /**
	                         11 byte array (byte order MSB first):
	                         byte[0][3]  speed;        = CAN speed [bps]
	                         byte[4]  prseg;           = CAN PRSEG register  (0-7)
	                         byte[5]   seg1ph;         = CAN SEG1PH register (0-7)
	                         byte[6]  seg2ph;          = CAN SEG1PH register (0-7)
	                         byte[7]  sjw;             = CAN SJW register (0-3)
	                         byte[8]   sam;            = CAN SAM register: 0=one_sample, 1=three_samples
	                         byte[9]   output;         = CAN output type: 0=signals, 1=messages
	                         byte[10]  input;        = CAN input type: 0=signals, 1=messages
	                      **/
    CMD_DTYPE_DIOCONFIG,  /**
	                         3 byte array (byte order MSB first):
	                         byte[0] idx;      = I/O idx
	                         byte[1] enable;   = I/O enable (0|1)
	                         byte[2] type;     = I/O type (0=input | 1=output)
	                      **/
    CMD_DTYPE_ADVANCED, /** 12 byte array  (byte order MSB first): \n
                         byte[0][1]                 = uartPort \n
                         byte[2][3]                 = canPort \n
                         byte[4][5]                 = d4ioPort \n
                         byte[6][7]                 = uart2WifiMinData \n
                         byte[8][9]                 = uart2WifiMaxWaitMs  \n
                         byte[10][11]                 = d4ioPollingTimeMs  \n
                         byte[12][44]               = initString
                         */
	CMD_DTYPE_CANDB_MSG_INPUT, /** 5 byte array  (byte order MSB first):
	                             byte[0][3]  msgId;          = CAN message - msg ID
	                             byte[4] msgIdExt;           = CAN message - msg IDtype (extended,standard)
                         */
    CMD_DTYPE_I2C, /** 7 byte array  (byte order MSB first): \n
                         uint32                 = frequency \n
                         uint8                  = outputType
                         uint16                 = offset */
    CMD_DTYPE_CANBUS_FILTER,     /**< CANBUS filter type, 0=normal, 1=controller-based */
    CMD_DTYPE_USERTASK,     /**<
                                        16bit user task scheduler execution, in seconds
                                        8bit auto reboot OFF/ON
                                    */
    CMD_DTYPE_POWERSAVE,     /**
                                    uint8   type;
                                    uint16  bootDelayTime;
                                    uint32  sleepTime;
                                    uint32  runningTime;
                                    uint8   ledOnSleep;
                                    */
    CMD_DTYPE_POWERSAVE_SLEEPNOW,     /**
                                        uint8   sleepNow;
                                        uint32  sleepNowTime;
                                    */
    CMD_DTYPE_POWERSAVE_SETRUNNINGCOUNTER,     /**
                                        uint32  runningCounter;
                                    */
    CMD_DTYPE_num_values		/**< Total number of data types available */

} t_wiicom_cmd_data_type;

/**
 *  @brief Command/message structure type
 */
typedef struct
{
    char head[2];       		/**< Opening sequence message ("ST") */
    uint8_t msgCnt[2];     		/**< progressive counter of sent messages */
    uint8_t dest;          		/**< Recipient ID message */
    uint8_t dataLength[2]; 		/**< Length in bytes of data to be transmitted */
    uint8_t action[2];     		/**< Action type */
    uint8_t data[WIICOM_CMD_MAX_BUFFER_DATA]; /**< Data frame that contains datas to be sent or to be received */
    uint8_t crc[2];     		/**< CRC16 of the message */
    char term[2];    			/**< Closing sequence message ("EN") */

    uint8_t step_elab;							/**< reserved */
    uint8_t buffer[WIICOM_CMD_MAX_BUFFER_DATA]; /**< reserved */ // buffer of data to be processed
    uint16_t idx_buffer; 						/**< reserved */ // buffer index of the data that I've processed
    uint16_t idx_stop_buffer; 					/**< reserved */ // indicating how far into the buffer is filled with data
    uint16_t idx_data; 							/**< reserved */ // index of how much data I copied from the buffer to the given structure (which contains the actual data of the command)
} t_wiicom_comm_msg;

/**
 *  @brief Struct definition for \ref CMD_DTYPE_SIZE
 */
typedef struct
{
    uint8_t bytes_size;			/**< Length of a data type */
    uint8_t max_data_count;		/**< Max data count of a data type */
} t_dtype_size;


/** @ingroup grp_dtype_size */
/** @{ */
/**
 *  @brief Array containing the length of each data type t_wiicom_cmd_data_type and the max data count
 *  The length of a data type can be obtained by: CMD_DTYPE_SIZE[CMD_DTYPE_IPADDR].bytes_size The result obtained in this example is 4
 *  The max data count of a data type can be obtained by: CMD_DTYPE_SIZE[CMD_DTYPE_IPADDR].max_data_count The result obtained in this example is 1
 */

extern const t_dtype_size CMD_DTYPE_SIZE[];
/** @} */

/**
 *  @brief Types of RF power level
 */
typedef enum
{
    RF_POWER_LOW,
    RF_POWER_MEDIUM,
    RF_POWER_HIGH
} t_compact_net_rf_power;

/**
 *  @brief Types of wireless data rate speed
 */
typedef enum
{
    DATA_RATE_AUTO,
    DATA_RATE_1Mbs,
    DATA_RATE_2Mbs,
    DATA_RATE_5P5Mbs,
    DATA_RATE_11Mbs,
    DATA_RATE_6Mbs,
    DATA_RATE_9Mbs,
    DATA_RATE_12Mbs
} t_compact_net_datarate;

/**
 *  @brief Struct definition for the Compact module network configuration
 */
typedef struct
{
    uint8_t nwtype;             /**< Compact module network type 0=AD-HOC, 1=INFRASTRUCTURE */
    uint8_t macaddress[6];      /**< Compact module MAC address */
    uint8_t ipaddress[4];       /**< Compact module IP address */
    uint8_t netmask[4];         /**< Compact module network mask */
    uint8_t gateway[4];         /**< Compact module gateway IP address */
    char ssid[33];              /**< Compact module SSID (to create or to connect) */
    uint8_t security_type;      /**<  0=OPEN, 1=WPA1, 2=WPA2, 3=WEP  */
    char psk[33];               /**< Password of infrastructure to connect */
    uint8_t channel;          /**< Compact module wi-fi channel (from 1 to 14) */
    t_compact_net_rf_power rf_power_level;         /**< Compact module network RF Power level */
	t_compact_net_datarate data_rate;    	/**< Compact module network data rate */

} t_wiicom_comm_compact_net;

/**
 *  @brief Struct definition for the Compact module UART configuration
 */
typedef struct
{
    uint32_t baudrate;          /**< Compact module UART baud rate */
    uint8_t stopbits;           /**< Compact module UART stop bits (1 or 2) */
    uint8_t parity;             /**< Compact module UART parity check (0=NONE, 1=ODD, 2=EVEN) */
} t_wiicom_comm_compact_uart;

/**
 *  @brief Struct definition for the Compact module information data
 */
typedef struct
{
    char version[33];               /**< Compact module version */
    char type;                      /**< Compact module type (S=Compact-S, C=Compact-CS */
    uint16_t internal_buffer_size;   /**< Compact module internal buffer size */
    char alias[33];                 /**< Compact module custom description/name */
} t_wiicom_comm_compact_info;


/**
 *  @brief Canbus output type
 */
#define CANBUS_OUTPUT_XML_SIGNALS_FILTER    0
#define CANBUS_OUTPUT_XML_MESSAGES_FILTER   1
#define CANBUS_OUTPUT_XML_ALL               2
#define CANBUS_OUTPUT_CSV_SIGNALS_FILTER    3
#define CANBUS_OUTPUT_CSV_MESSAGES_FILTER   4
#define CANBUS_OUTPUT_CSV_ALL               5
#define CANBUS_OUTPUT_DISABLED              8

/**
 *  @brief Canbus input type
 */
 #define CANBUS_INPUT_CSV_ALL               0
 #define CANBUS_INPUT_CSV_MESSAGES_FILTER   1
 #define CANBUS_INPUT_DISABLED              2

typedef enum
{
    CSV_ALL,
    CSV_MESSAGES_FILTER,
    DISABLED
} t_wiicom_cmd_canbus_input;


/**
 *  @brief Function that cleans up the data communication structure
 *  @param[in]  b_reset_all if set to TRUE cleans the entire structure, if FALSE only the data it contains
 *  leaving unchanged the accessories data used internally for the operations of command decoding
 *  @param[in,out]  cmd_msg communication data structure to clean
 *  @return void
 */
void wiicom_comm_reset(int8_t b_reset_all,t_wiicom_comm_msg *cmd_msg);

/**
 *  @brief Function that prints the contents of the communication data structure
 *  @param[in]  fp pointer files on which to perform printing of the data
 *  @param[in]  cmd_msg communication data structure that you want to print
 *  @return void
 */
void  wiicom_comm_print(FILE *fp,t_wiicom_comm_msg *cmd_msg);

/**
 *  @brief Function that extracts datas from a buffer messages/commands sent by your Compact module
 *  @param[in]  rxData buffer containing the messages/commands to be interpreted
 *  @param[in]  num_rxData number of bytes to interpret
 *  @param[in,out]  cmd_msg communication data structure to populate
 *  @param[in]  callback function is invoked when a valid command is extracted from the buffer\n @b Caution: If set to NULL will not be executed
 *  @param[in]  callback_error function is invoked when an error occurred on extraction of a command from the buffer\n @b Caution: If set to NULL will not be executed
 *  @return void
 */
void wiicom_comm_elab(uint8_t *rxData,uint32_t num_rxData,t_wiicom_comm_msg *cmd_msg,
                      void (*callback)(t_wiicom_comm_msg*), void (*callback_error)(t_wiicom_comm_msg*));

/**
 *  @brief Function that initializes the communication data structure for send a new command to a Compact module
 *  @param[in,out]  cmd_msg command data structure to be initialized
 *  @param[in]  par_msg_cnt number of the message to be sent
 *  @param[in]  par_destination number that identifies the recipient
 *  @param[in]  par_action action that is required to be performed by the Compact module
 *  @return void
 */
void wiicom_comm_createCommand(t_wiicom_comm_msg *cmd_msg,uint16_t par_msg_cnt,
                               uint8_t par_destination,uint16_t par_action);

/**
 *  @brief Function that adds a datablock to a given communication data structure
 *  @param[in,out]  cmd_msg communication structure to add a given datablock
 *  @param[in]  par_type type of data added
 *  @param[in]  par_data_cnt data vector number elements to be add; set it only for vector data type, otherwise 1
 *  @param[in]  par_data_idx data vector index (zero based start index); set it only for vector data type, otherwise 0
 *  @param[in]  par_data buffer containing the data to be added
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_addToDataBlock(t_wiicom_comm_msg *cmd_msg,uint16_t par_type,
                                   uint8_t par_data_cnt,uint8_t par_data_idx,uint8_t *par_data);

/**
 *  @brief Function which finalizes the communication data structure, the command is ready to be sent to your Compact
 *  @param[in,out]  cmd_msg communication data structure to be finalized
 *  @return void
 */
void wiicom_comm_closeCommand(t_wiicom_comm_msg *cmd_msg);

/**
 *  @brief Function that extracts a datablock from communication data structure
 *  @param[in]  cmd_msg communication data structure from which you want to extract datablock
 *  @param[in]  par_type type of data to be extracted
 *  @param[in]  par_data_cnt data vector number elements to be extracted, otherwise 1
 *  @param[in]  par_data_idx  data vector index (zero based start index); it sets only for vector data type, otherwise 0
 *  @param[out]  par_data it contains the data extracted
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_getDataBlock(t_wiicom_comm_msg *cmd_msg,uint16_t par_type,
                                 uint8_t par_data_cnt,uint8_t par_data_idx,uint8_t *par_data);

/**
 *  @brief Function that converts the content of a communication data structure in a sequence of bytes
 *  @param[in]  cmd_msg communication data structure on which you want to perform the conversion
 *  @param[out]  rowData that will contain the binary conversion of the communication data structure
 *  @param[out]  rowData_length that will contain the bytes length of binary conversion
 *  @return void
 */
void wiicom_comm_cmd_msgTOrow(t_wiicom_comm_msg *cmd_msg,uint8_t* rowData,uint16_t *rowData_length);

/**
 *  @brief function that checks if a command/message has been received completely
 *  @param[in]  cmd_msg communication data structure on which you want to perform the check
 *  @return  On a complete recived command/message, TRUE is returned, otherwise FALSE is returned
 */
uint8_t wiicom_comm_recivIScomplete(t_wiicom_comm_msg *cmd_msg);

/**
 *  @brief Function that sends a command to a Compact module and, if provided, returns the response
 *  @param[in,out]  cmd_msg communication data structure to be sent to your Compact, after sending the structure will contain the answer of your Compact
 *  @param[in]  ip module IP address to send the command
 *  @param[in]  port Module port to send the command
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_sendCommand(t_wiicom_comm_msg *cmd_msg,const char *ip,uint16_t port);

/**
 *  @brief Function that sends a command to a Compact module on pre-open socket and, if provided, returns the response
 *  @param[in,out]  cmd_msg communication data structure to be sent to your Compact, after sending the structure will contain the answer of your Compact
 *  @param[in]  socketfd a file descriptor for the socket
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_sendCommand_socket(t_wiicom_comm_msg *cmd_msg,int socketfd);

/**
 *  @brief Function that read Compact module information
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[out]  info_struct info data structure that will contain Compact module information
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_get_compact_info(const char *ip,uint16_t port,t_wiicom_comm_compact_info *info_struct);

/**
 *  @brief Function that set Compact module alias
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[in]  alias string that contain the new Compact module alias
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_set_compact_alias(const char *ip,uint16_t port,char *alias);

/**
 *  @brief Function that read Compact module UART configuration
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[out]  uart_struct UART configuration data structure that will contain Compact module UART config
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_get_compact_uart(const char *ip,uint16_t port,t_wiicom_comm_compact_uart *uart_struct);
/**
 *  @brief Function that set Compact module UART configuration
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[in]  uart_struct UART configuration data structure that will be set on Compact module
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_set_compact_uart(const char *ip,uint16_t port,t_wiicom_comm_compact_uart *uart_struct);

/**
 *  @brief Function that read Compact module network configuration
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[out]  net_struct network configuration data structure that will contain Compact module network config
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_get_compact_net(const char *ip,uint16_t port,t_wiicom_comm_compact_net *net_struct);
/**
 *  @brief Function that set Compact module network configuration
 *  @note @b Psk is save only when @b security_type is not 0. \n @b Gateway is saved only when @b nwtype is not 0 (AD-HOC).
 *  @param[in]  ip module IP address
 *  @param[in]  port module port
 *  @param[in]  net_struct network configuration data structure that will be set on Compact module
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_comm_set_compact_net(const char *ip,uint16_t port,t_wiicom_comm_compact_net *net_struct);






//NUOVE FUNZIONI DA COMMENTARE

uint8_t wiicom_comm_get_compact_dio(const char *ip,uint16_t port,uint8_t num_dio,uint8_t *b_enabled,uint8_t *type);
uint8_t wiicom_comm_set_compact_dio(const char *ip,uint16_t port,uint8_t num_dio,uint8_t b_enabled,uint8_t type);
uint8_t wiicom_comm_get_compact_canbus(const char *ip,uint16_t port,uint32_t *speed,uint8_t *output_type,uint8_t *input_type);
uint8_t wiicom_comm_set_compact_canbus(const char *ip,uint16_t port,uint32_t speed,uint8_t output_type,uint8_t input_type);
uint8_t wiicom_comm_sendMsgCanbus(const char *ip,uint16_t port,uint32_t idmsg,uint8_t b_extendedMsg,uint8_t dlc, uint8_t *data);
uint8_t wiicom_comm_sendMsgCanbus_socket(int socketfd,uint32_t idmsg,uint8_t b_extendedMsg,uint8_t dlc, uint8_t *data);
uint8_t wiicom_comm_parseMsgCanbus(const char *recivData,uint16_t recivedDataLength,uint32_t *idmsg,uint8_t *b_extendedMsg,uint8_t *dlc, uint8_t *data);





#endif
