/** @file wiicom_libsocket.c
 *  @brief Socket function library
 *
 *  Generic library for managing socket
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.0.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @attention This library depends on @b wiicom_libutil \n
 *  Every function that expects a result, in case of an error, set the Wiicom errno and print to stderr the error occurred.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */


#include "wiicom_libsocket.h"
#include "wiicom_libutil.h"

static uint8_t glb_sec_timeout_connect=SEC_TIMEOUT_CONNECT_SOCKET;
static uint8_t glb_sec_timeout_reciv=SEC_TIMEOUT_RECIV_SOCKET;

//========================================= FUNZIONI SPECIFICHE WINSOCK - INIZIO =========================================================================

#if (defined (__WIN32__))

typedef int socklen_t;

static void loc_socket_win_msg(errCode)
{
    LPSTR errString = NULL;
    int size = FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
                            0,errCode,0,(LPSTR)&errString,0,0 );
    if(size>0)  wiicom_util_set_err_msg(errString);
    else  wiicom_util_set_err_msg("Generic Windows socket error");

    LocalFree( errString ) ;
}

#endif
//========================================= FUNZIONI SPECIFICHE WINSOCK - FINE =========================================================================

uint8_t wiicom_socket_initLibSocket()
{
#if (defined (__WIN32__))
    WORD wVersionRequested;
    WSADATA wsaData;
    int wsaerr;

    // Using MAKEWORD macro, Winsock version request 2.2
    wVersionRequested = MAKEWORD(2, 2);

    wsaerr = WSAStartup(wVersionRequested, &wsaData);
    if (wsaerr != 0)
    {
        /* Tell the user that we could not find a usable */
        /* WinSock DLL.*/
		wiicom_util_set_errno(WIICOM_EWINSOCKETFAILURE);
        wiicom_util_perror("The Winsock dll not found!\n");
        return(FALSE);
    }

    /* Confirm that the WinSock DLL supports 2.2.*/
    /* Note that if the DLL supports versions greater    */
    /* than 2.2 in addition to 2.2, it will still return */
    /* 2.2 in wVersion since that is the version we      */
    /* requested.                                        */
    if (LOBYTE(wsaData.wVersion) != 2 || HIBYTE(wsaData.wVersion) != 2 )
    {
        char str_tmp[256]="";
        /* Tell the user that we could not find a usable */
        /* WinSock DLL.*/
        sprintf(str_tmp,"The dll do not support the Winsock version %u.%u!", LOBYTE(wsaData.wVersion),HIBYTE(wsaData.wVersion));
		wiicom_util_set_errno(WIICOM_EWINSOCKETFAILURE);
        wiicom_util_perror(str_tmp);

        WSACleanup();
        return(FALSE);
    }
#endif

    return(TRUE);
}

void wiicom_socket_deinitLibSocket()
{
#if (defined (__WIN32__))
    WSACleanup();
#endif
}

void wiicom_socket_set_timeout_conn(uint8_t sec)
{
	glb_sec_timeout_connect=sec;
}

void wiicom_socket_set_timeout_reciv(uint8_t sec)
{
	glb_sec_timeout_reciv=sec;
}

uint8_t wiicom_socket_get_timeout_conn()
{
	return(glb_sec_timeout_connect);
}

uint8_t wiicom_socket_get_timeout_reciv()
{
	return(glb_sec_timeout_reciv);
}

void wiicom_socket_set_address(struct sockaddr_in *socket_address,const char *ip,int port)
{
	socket_address->sin_family = AF_INET;
	socket_address->sin_port = htons(port);
	if(strcmp(ip,"")==0)
		socket_address->sin_addr.s_addr = htonl(INADDR_ANY);
	else
		socket_address->sin_addr.s_addr = inet_addr(ip);

	memset(socket_address->sin_zero, '\0', sizeof(socket_address->sin_zero));
}

uint8_t wiicom_socket_blocking(int socket_fd,uint8_t b_block)
{

#if (defined (__WIN32__))
    unsigned long arg;

    if(b_block) arg=0;
    else arg=1;

    if (0 != ioctlsocket(socket_fd, FIONBIO, &arg))
    {
            wiicom_util_set_errno(WSAGetLastError());
            loc_socket_win_msg(WSAGetLastError());
            return(FALSE);
    }
#else
    long arg;
    // Set non-blocking
    if( (arg = fcntl(socket_fd, F_GETFL, NULL)) < 0)
    {
		wiicom_util_set_errno(errno);
        return(FALSE);
    }

    if(b_block) arg &= (~O_NONBLOCK);
    else    arg |= O_NONBLOCK;

    if( fcntl(socket_fd, F_SETFL, arg) < 0)
    {
		wiicom_util_set_errno(errno);
        return(FALSE);
    }
#endif
    return(TRUE);
}

 int wiicom_socket_create(int domain, int type, int protocol)
 {
	 int socket_fd=0;
	 
	 if((socket_fd = socket(domain, type, protocol)) == -1)
	 {
	 	#if (defined (__WIN32__))
	 		wiicom_util_set_errno(WSAGetLastError());
	 		loc_socket_win_msg(WSAGetLastError());
	 	#else
	 		wiicom_util_set_errno(errno);
	 	#endif
	 }

 	return(socket_fd);
 }

void wiicom_socket_close(int socket_fd)
{
#if (defined (__WIN32__))
    closesocket(socket_fd);
#else
    close(socket_fd);
#endif
}

int wiicom_socket_connect(int socketToConnect, struct sockaddr_in *socketToConnect_addr)
{
    int res=-1;

    fd_set myset;
    struct timeval tv;
    int valopt;
    socklen_t lon;

    // Set non-blocking
    if(wiicom_socket_blocking(socketToConnect,FALSE)==FALSE)    return(-1);

    // Trying to connect with timeout
    res = connect(socketToConnect, (struct sockaddr *)socketToConnect_addr, sizeof(struct sockaddr));
    if (res < 0)
    {
        #if (defined (__WIN32__))
        if (WSAGetLastError() == WSAEINPROGRESS || WSAGetLastError() == WSAEWOULDBLOCK)
        #else
        if (errno == EINPROGRESS)
        #endif
        {
            do
            {
                tv.tv_sec = glb_sec_timeout_connect;
                tv.tv_usec = 0;
                FD_ZERO(&myset);
                FD_SET(socketToConnect, &myset);
                res = select(socketToConnect+1, NULL, &myset, NULL, &tv);
                #if (defined (__WIN32__))
                if (res < 0 && WSAGetLastError() != WSAEINTR)
                #else
                if (res < 0 && errno != EINTR)
                #endif
                {
                    #if (defined (__WIN32__))
                    wiicom_util_set_errno(WSAGetLastError());
                    loc_socket_win_msg(WSAGetLastError());
                    #else
                    wiicom_util_set_errno(errno);
                    #endif
                    return(-1);
                }
                else if (res > 0)
                {
                    // Socket selected for write
                    lon = sizeof(int);
                    if (getsockopt(socketToConnect, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon) < 0)
                    {
						wiicom_util_set_errno(errno);
                        return(-1);
                    }
                    // Check the value returned...
                    if (valopt)
                    {
                        #if (defined (__WIN32__))
                        wiicom_util_set_errno(WSAGetLastError());
                        loc_socket_win_msg(WSAGetLastError());
                        #else
                        wiicom_util_set_errno(errno);
                        #endif
                        return(-1);
                    }
                    break;
                }
                else
                {
					wiicom_util_set_errno(WIICOM_ESOCKETCONNTIMEOUT);
                    res=-1;
                    break;
                }
            } while (1);
        }
        else
        {
            #if (defined (__WIN32__))
            wiicom_util_set_errno(WSAGetLastError());
            loc_socket_win_msg(WSAGetLastError());
            #else
            wiicom_util_set_errno(errno);
            #endif
            res=-1;
        }
    }

    // Set to blocking mode again...
    if(wiicom_socket_blocking(socketToConnect,TRUE)==FALSE)    return(-1);

    return(res);
}

int wiicom_socket_sendall(int socket_fd, uint8_t *buf, uint16_t *len)
{
	int total = 0; // quanti byte abbiamo inviato
	int bytesleft = *len; // quanti ne rimangono da inviare
	ssize_t n;

	while(total < *len)
	{
	    #if (defined (__WIN32__))
		n = send(socket_fd, (char *)(buf+total), bytesleft, 0); if (n == -1) { break; }
	    #else
		n = send(socket_fd, buf+total, bytesleft, 0); if (n == -1) { break; }
	    #endif

		total += n;
		bytesleft -= n;
	}
    if(n==-1)
    {
        #if (defined (__WIN32__))
                wiicom_util_set_errno(WSAGetLastError());
                loc_socket_win_msg(WSAGetLastError());
        #else
                wiicom_util_set_errno(errno);
        #endif
    }
	*len = total; // ritorna il numero effettivamente inviato
	return(n==-1?-1:0); // restituisce -1 in caso di fallimento, 0 in caso di successo
}

ssize_t wiicom_socket_reciv(int socket_fd,void *buffer,size_t buffer_size)
{
	fd_set set;
	struct timeval timeout;
	ssize_t numbytes=0;
	int fl_dati_in_arrivo=0;

	FD_ZERO(&set);
	FD_SET(socket_fd, &set);

	timeout.tv_sec = glb_sec_timeout_reciv;
	timeout.tv_usec = 0;

    memset(buffer,'\0',buffer_size);

	fl_dati_in_arrivo = select(socket_fd + 1, &set, NULL, NULL, &timeout);
    #if (defined (__WIN32__))
	if(fl_dati_in_arrivo == -1 && WSAGetLastError()!=WSAEINTR)
    #else
	if(fl_dati_in_arrivo == -1 && errno!=EINTR)
    #endif
	{
        #if (defined (__WIN32__))
        wiicom_util_set_errno(WSAGetLastError());
        loc_socket_win_msg(WSAGetLastError());
        #else
        wiicom_util_set_errno(errno);
        #endif
		wiicom_socket_close(socket_fd);
		return(-1);
	}
	else if(fl_dati_in_arrivo == 0)
	{
		// timeout chiudo il socket
		wiicom_util_set_errno(WIICOM_ESOCKETRECVTIMEOUT);
		wiicom_socket_close(socket_fd);
		return(-1);
	}
	else if(fl_dati_in_arrivo > 0)
	{
		numbytes=recv(socket_fd, buffer, buffer_size, 0);

		if(numbytes==-1)
		{
	        #if (defined (__WIN32__))
	        wiicom_util_set_errno(WSAGetLastError());
	        loc_socket_win_msg(WSAGetLastError());
	        #else
			wiicom_util_set_errno(errno);
			#endif
			wiicom_socket_close(socket_fd);
		}
		else if(numbytes==0)
		{
			//connessione chiusa
			wiicom_socket_close(socket_fd);
		}
	}

	return(numbytes);
}

ssize_t wiicom_socket_notimeout_reciv(int socket_fd,void *buffer,size_t buffer_size)
{
	ssize_t numbytes=0;
	numbytes=recv(socket_fd, buffer, buffer_size, 0);

	if(numbytes==-1)
	{
        #if (defined (__WIN32__))
        wiicom_util_set_errno(WSAGetLastError());
        loc_socket_win_msg(WSAGetLastError());
        #else
		wiicom_util_set_errno(errno);
		#endif
		wiicom_socket_close(socket_fd);
	}
	else if(numbytes==0)
	{
		//connessione chiusa
		wiicom_socket_close(socket_fd);
	}
	
	return(numbytes);
}

uint8_t wiicom_socket_shutdown(int socket_fd)
{
 	#if (defined (__WIN32__))
	if(shutdown(socket_fd,2)!=0)
	{
		wiicom_util_set_errno(WSAGetLastError());
		loc_socket_win_msg(WSAGetLastError());
		return(FALSE);
	} 	
 	#else
	if(shutdown(socket_fd,SHUT_RDWR)!=0)
	{
		wiicom_util_set_errno(errno);
		return(FALSE);
	} 	
	#endif
	
	return(TRUE);
}