/** @file wiicom_libsocket.h
 *  @brief Socket function library
 *
 *  Generic library for managing socket
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.0.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @attention This library depends on @b wiicom_libutil \n
 *  Every function that expects a result, in case of an error, set the Wiicom errno and print to stderr the error occurred.
 *  @note None.
 *  @todo 
 *  @deprecated
 *  @bug No bugs found.
 */

#ifndef _WIICOM_LIBSOCKET_H
#define _WIICOM_LIBSOCKET_H
 
#if (defined (__WIN32__))
	#pragma comment(lib,  "ws2_32.lib") 

    //#include <winsock2.h>
    #include <windows.h>
	#include <stdint.h>
	
	#define ssize_t SSIZE_T

#else
    #include <sys/time.h>
    #include <sys/types.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <netdb.h>
    #include <arpa/inet.h>
    #include <sys/select.h>
    #include <fcntl.h>
#include <unistd.h>
#endif //if (defined (__WIN32__))


#define SEC_TIMEOUT_CONNECT_SOCKET	5	/**< Number of seconds before a timeout error during the connection phase */
#define SEC_TIMEOUT_RECIV_SOCKET	10	/**< Number of seconds before a timeout error during data reception */

/**
 *  @brief Function that initializes the socket library\n
 *  @b Warning! This function should be called before using the socket.
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_socket_initLibSocket();

/**
 *  @brief Function that de-initialize the socket library\n
 *  @b Warning! This function is called when not used any more a sockets or, in general, before the close of the program.
 *  @return void
 */
void wiicom_socket_deinitLibSocket();

/**
 *  @brief Function that sets the connection timeout of a socket
 *  @param[in]  sec  number of seconds
 *  @return void
 */
void wiicom_socket_set_timeout_conn(uint8_t sec);

/**
 *  @brief Funzione che imposta il timeout della ricezione dati di un socket
 *  @param[in]  sec  number of seconds
 *  @return void
 */
void wiicom_socket_set_timeout_reciv(uint8_t sec);

/**
 *  @brief Function that returns the seconds after which there is a timeout in the connection of a socket
 *  @return Timeout in seconds
 */
uint8_t wiicom_socket_get_timeout_conn();

/**
 *  @brief Function that returns the seconds after which you have a timeout when receiving data
 *  @return Timeout in seconds
 */
uint8_t wiicom_socket_get_timeout_reciv();

/**
 *  @brief Funzione che imposta il socket come bloccante o non bloccante
 *  @param[in]  socket_fd  a file descriptor for the socket
 *  @param[in]  b_block if TRUE sets the socket as a blocker, if FALSE sets it as non-blocking
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_socket_blocking(int socket_fd,uint8_t b_block);

/**
 *  @brief Function that set a socket address
 *  @param[out]  socket_address socket_address to set
 *  @param[in]  ip  IP address
 *  @param[in]  port  Port
 *  @return void
 */
void wiicom_socket_set_address(struct sockaddr_in *socket_address,const char *ip,int port);

/**
 *  @brief Function that create a socket
 *  @param[in]  domain  communication domain 
 *  @param[in]  type  specifies the communication semantics
 *  @param[in]  protocol  specifies a particular protocol to be used with the socket
 *  @return on success, a file descriptor for the new socket is returned. On error, -1 is returned
 */
int wiicom_socket_create(int domain, int type, int protocol);

/**
 *  @brief Function that close a socket
 *  @param[in]  socket_fd  a file descriptor for the socket to close
 *  @return void
 */
void wiicom_socket_close(int socket_fd);

/**
 *  @brief Function that make a socket connection with timeout
 *  @param[in]  socketToConnect  a file descriptor for the socket on which connect
 *  @param[in]  socketToConnect_addr  sockaddr_in
 *  @return  On success, 0 is returned, On error, -1 is returned
 */
int wiicom_socket_connect(int socketToConnect, struct sockaddr_in *socketToConnect_addr);

/**
 *  @brief Function that sends a data buffer on socket
 *  @param[in]  socket_fd   a file descriptor for the socket where datas are send
 *  @param[in]  buf  a datas to send
 *  @param[in,out]  len number of bytes in the buffer to be sent, after sending it contains the number of bytes sent
 *  @return  On success, 0 is returned. On error, -1 is returned
 */
int wiicom_socket_sendall(int socket_fd, uint8_t *buf, uint16_t *len);

/**
 *  @brief Function that receives the data transmitted to the socket with timeout
 *  On error or timeout the socket is closed.
 *  @param[in]  socket_fd  a file descriptor for the socket where datas are recived
 *  @param[out]  buffer  it contains data to send
 *  @param[in]  buffer_size  size of the buffer to send
 *  @return On success, number of bytes transmitted. 0 is returned if the socket has been closed. On error, -1 is returned
 */
ssize_t wiicom_socket_reciv(int socket_fd,void *buffer,size_t buffer_size);

/**
 *  @brief Function that receives the data transmitted to the socket without timeout
 *  @param[in]  socket_fd  a file descriptor for the socket where datas are recived
 *  @param[out]  buffer  buffer in which datas are loaded
 *  @param[in]  buffer_size  size of the buffer containing the data read
 *  @return On success, number of bytes received. 0 is returned if the socket has been closed. On error, -1 is returned
 */
ssize_t wiicom_socket_notimeout_reciv(int socket_fd,void *buffer,size_t buffer_size);


/**
 *  @brief Function that shut down a socket
 *  @param[in]  socket_fd  a file descriptor for the socket to close
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_socket_shutdown(int socket_fd);

#endif

