/** @file wiicom_libutil.c
 *  @brief Wiicom Virtucal com port library
 *
 *  Generic library for error handling, data conversion and calculation of CRC16 (Modbus standard CRC)
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.0.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */

#include "wiicom_libutil.h"

#define WIICOM_LEN_CUSTOM_ERR_MSG 256

//========================================= GESTIONE ERRORI - INIZIO =========================================================================
static int16_t wiicom_util_errno=0;
static char wiicom_custom_err_msg[WIICOM_LEN_CUSTOM_ERR_MSG];

const char *wiicom_sys_errlist[] = {
	"",
	"winsock init failure",						                    // -1
	"socket connection timeout",						            // -2
	"socket reciv timeout",								            // -3
	"datablock not added",								            // -4
	"datablock not found",								            // -5
	"elab command message failure",						            // -6
	"invalid virtual com port baudrate",				            // -7
	"invalid virtual com port data size",				            // -8
	"unable to save virtual com port attributes",		            // -9
	"invalid virtual com port parity check",			            // -10
	"invalid Compact Module Alias",                                 // -11
	"invalid Compact Module Network configuration",		            // -12
	"invalid Compact Module UART configuration",                    // -13
	"function is not supported for the selected Compact Module"     // -14

};

const char *wiicom_util_strerror(int16_t par_errnum)
{
    if(wiicom_custom_err_msg[0]!='\0')
        return(wiicom_custom_err_msg);
    else
    {
        if(par_errnum<0)
        {
            if(par_errnum<WIICOM_N_ERRNO) return("");
            else	return(wiicom_sys_errlist[par_errnum*-1]);
        }
        else
            return(strerror(par_errnum));
    }
}

void wiicom_util_perror(const char *string)
{
	char *output_string;

	output_string=(char *)wiicom_util_strerror(wiicom_util_errno);
	if(output_string[0]=='\0')
	{
		fprintf(stderr,"%s: Undefined error: %d\n",string,wiicom_util_errno);
	}
	else
	{
		fprintf(stderr,"%s: %s\n",string,output_string);
	}
}

void wiicom_util_set_errno(int32_t par_errnum)
{
    memset((void*)wiicom_custom_err_msg,'\0',WIICOM_LEN_CUSTOM_ERR_MSG);
	wiicom_util_errno=par_errnum;
}

int32_t wiicom_util_get_errno()
{
	return(wiicom_util_errno);
}

void wiicom_util_set_err_msg(const char *string)
{
    strncpy(wiicom_custom_err_msg,string,WIICOM_LEN_CUSTOM_ERR_MSG);
    wiicom_custom_err_msg[WIICOM_LEN_CUSTOM_ERR_MSG-1]='\0';
}
//========================================= GESTIONE ERRORI - FINE =========================================================================


/* Table of CRC values for high-order byte */
static uint8_t tabCrcHi[256] = {
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
	0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
	0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
	0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81,
	0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
	0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
	0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
	0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
	0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
	0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
	0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
	0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
	0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
	0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
	0x40
} ;

/* Table of CRC values for low-order byte */
static uint8_t tabCrcLo[256] = {
	0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06, 0x07, 0xC7, 0x05, 0xC5, 0xC4,
	0x04, 0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09,
	0x08, 0xC8, 0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A, 0x1E, 0xDE, 0xDF, 0x1F, 0xDD,
	0x1D, 0x1C, 0xDC, 0x14, 0xD4, 0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3,
	0x11, 0xD1, 0xD0, 0x10, 0xF0, 0x30, 0x31, 0xF1, 0x33, 0xF3, 0xF2, 0x32, 0x36, 0xF6, 0xF7,
	0x37, 0xF5, 0x35, 0x34, 0xF4, 0x3C, 0xFC, 0xFD, 0x3D, 0xFF, 0x3F, 0x3E, 0xFE, 0xFA, 0x3A,
	0x3B, 0xFB, 0x39, 0xF9, 0xF8, 0x38, 0x28, 0xE8, 0xE9, 0x29, 0xEB, 0x2B, 0x2A, 0xEA, 0xEE,
	0x2E, 0x2F, 0xEF, 0x2D, 0xED, 0xEC, 0x2C, 0xE4, 0x24, 0x25, 0xE5, 0x27, 0xE7, 0xE6, 0x26,
	0x22, 0xE2, 0xE3, 0x23, 0xE1, 0x21, 0x20, 0xE0, 0xA0, 0x60, 0x61, 0xA1, 0x63, 0xA3, 0xA2,
	0x62, 0x66, 0xA6, 0xA7, 0x67, 0xA5, 0x65, 0x64, 0xA4, 0x6C, 0xAC, 0xAD, 0x6D, 0xAF, 0x6F,
	0x6E, 0xAE, 0xAA, 0x6A, 0x6B, 0xAB, 0x69, 0xA9, 0xA8, 0x68, 0x78, 0xB8, 0xB9, 0x79, 0xBB,
	0x7B, 0x7A, 0xBA, 0xBE, 0x7E, 0x7F, 0xBF, 0x7D, 0xBD, 0xBC, 0x7C, 0xB4, 0x74, 0x75, 0xB5,
	0x77, 0xB7, 0xB6, 0x76, 0x72, 0xB2, 0xB3, 0x73, 0xB1, 0x71, 0x70, 0xB0, 0x50, 0x90, 0x91,
	0x51, 0x93, 0x53, 0x52, 0x92, 0x96, 0x56, 0x57, 0x97, 0x55, 0x95, 0x94, 0x54, 0x9C, 0x5C,
	0x5D, 0x9D, 0x5F, 0x9F, 0x9E, 0x5E, 0x5A, 0x9A, 0x9B, 0x5B, 0x99, 0x59, 0x58, 0x98, 0x88,
	0x48, 0x49, 0x89, 0x4B, 0x8B, 0x8A, 0x4A, 0x4E, 0x8E, 0x8F, 0x4F, 0x8D, 0x4D, 0x4C, 0x8C,
	0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42, 0x43, 0x83, 0x41, 0x81, 0x80,
	0x40
};

void wiicom_util_sleep(uint32_t millisec)
{
	#if (defined (__WIN32__))
	Sleep(millisec);
	#else
	usleep(millisec*1000);
	#endif
}

uint8_t wiicom_util_isBigEndianSystem()
{
    int num = 1;
    if(*(char *)&num == 1)   return(FALSE);
    else   return(TRUE);
}

uint16_t wiicom_util_endianU16Conversion(uint16_t word)
{
	return ((word>>8)&0x00FF) | ((word<<8)&0xFF00)  ;
}

uint32_t wiicom_util_endianU32Conversion(uint32_t dword)
{
	return ((dword>>24)&0x000000FF) | ((dword>>8)&0x0000FF00) | ((dword<<8)&0x00FF0000) | ((dword<<24)&0xFF000000);
}

uint16_t wiicom_util_CRC16(uint8_t * msgByte, uint16_t msgByteLength)
{
	uint8_t crcHi = 0xFF; 	/*crc Hi byte init*/
	uint8_t crcLo = 0xFF; 	/*crc Lo byte init*/
	uint8_t tabIdx; 			/*crc lookup-table index*/

	/* cycle over msg bytes */
	while (msgByteLength--)
	{
		/* crc calculation */
		tabIdx = crcLo ^ *msgByte++;
		crcLo = crcHi ^ tabCrcHi[tabIdx];
		crcHi = tabCrcLo[tabIdx];
	}

	return ( (crcHi << 8) | crcLo);
}

uint16_t wiicom_util_walkingCRC16(uint8_t msgByte, uint8_t msgStart)
{
    static uint8_t crcHi = 0xFF; 	/*crc Hi byte */
	static uint8_t crcLo = 0xFF; 	/*crc Lo byte */
	static uint8_t tabIdx = 0; 	/*crc lookup-table index*/

    if (msgStart==1)
    {
        crcHi = 0xFF; 	/*crc Hi byte init*/
	    crcLo = 0xFF; 	/*crc Lo byte init*/
    }

	/* crc calculation */
	tabIdx = crcLo ^ msgByte;
	crcLo = crcHi ^ tabCrcHi[tabIdx];
	crcHi = tabCrcLo[tabIdx];

	return ( (crcHi << 8) | crcLo);
}

