/** @file wiicom_libutil.h
 *  @brief Wiicom Virtucal com port library
 *
 *  Generic library for error handling, data conversion and calculation of CRC16 (Modbus standard CRC)
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.0.1
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */

#ifndef _WIICOM_LIBUTIL_H
#define _WIICOM_LIBUTIL_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <errno.h>

#ifndef __WIN32__
	#include <unistd.h>
#else
	#include <windows.h>
#endif

/** @defgroup boolean Boolean definition
 * @brief The definition of the Boolean values true and false is present in wiicom_libutil.h
 * @{ */
#define FALSE 0	/**< Definition of the boolean value false */
#define TRUE 1	/**< Definition of the boolean value true */
/** @} */

//========================================= ERROR MANAGEMENT - START =========================================================================

/** @defgroup errnum Wiicom equivalent errno
 * @brief Wiicom equivalent errno
 *
 * The definition of Wiicom errno is present in wiicom_libutil.h
 * @{ */
#define WIICOM_EWINSOCKETFAILURE		-1	/**< Error while creating a socket */
#define WIICOM_ESOCKETCONNTIMEOUT		-2	/**< Timeout during connection to a socket */
#define WIICOM_ESOCKETRECVTIMEOUT		-3	/**< Timeout while receiving data from a socket */
#define WIICOM_EADDDATABLOCK            -4	/**< Error adding a datablock to a Compact message */
#define WIICOM_EGETDATABLOCK            -5	/**< Error retrieving a datablock from a Compact message */
#define WIICOM_EELABCOMM_MSG            -6	/**< Error while parsing the structure of a Compact message */
#define WIICOM_EVCPBAUD					-7	/**< Error while set a unsupported vcp baud rate */
#define WIICOM_EVCPDATASIZE				-8	/**< Error while set a unsupported vcp data size */
#define WIICOM_EVCPATTR					-9	/**< Error while save virtual com port old attributes */
#define WIICOM_EVCPPARITY               -10 /**< Error while set a unsupported vcp parity check */
#define WIICOM_ESETCMP_ALIAS            -11 /**< Parameter error while set Compact Module alias */
#define WIICOM_ESETCMP_NET              -12 /**< Parameter error while set Compact Module network configuration */
#define WIICOM_ESETCMP_UART             -13 /**< Parameter error while set Compact Module UART configuration */
#define WIICOM_ENOTSUPPORTED			-14	/**< Error while you call a function that is not supported by the Compact */
#define WIICOM_N_ERRNO 					-15	/**< Maximum Wiicom Errno that can be generated from Wiicom libraries */
/** @} */

/**
 *  @brief Function that prints to stderr the description of the last error found
 *  @param[in]  string  Text to prepend to the error description
 *  @return void
 */
void wiicom_util_perror(const char *string);

/**
 *  @brief Function that returns the description for the error number passed as a parameter
 *  @param[in]  par_errnum  Error number you want to have the description
 *  @return string containing the error description
 */
const char *wiicom_util_strerror(int16_t par_errnum);


/**
 *  @brief Function that sets the errno in the system error handling
 *  @param[in]  par_errnum  Error number to set
 *  @return void
 */
void wiicom_util_set_errno(int32_t par_errnum);


/**
 *  @brief Function that returns the last errno reported
 *  @return void
 */
int32_t wiicom_util_get_errno();


/**
 *  @brief Function that override the error description returned by wiicom_util_perror
 *  @param[in]  string  string to be used for overwriting
 *  @return void
 */
void wiicom_util_set_err_msg(const char *string);
//========================================= ERROR MANAGEMENT - END =========================================================================

/**
 *  @brief Function that checks if the system is Big Endian
 *  @return TRUE if Big Endian, Little Endian if FALSE
 */
uint8_t wiicom_util_isBigEndianSystem();

/**
 *  @brief Function that reverses the order of the bytes of a uint16_t
 *  @param[in]  word  The number to convert
 *  @return Number with the bytes reversed
 */
uint16_t wiicom_util_endianU16Conversion(uint16_t word);

/**
 *  @brief Function that reverses the order of the bytes of a uint32_t
 *  @param[in]  dword  The number to convert
 *  @return Number with the bytes reversed
 */
uint32_t wiicom_util_endianU32Conversion(uint32_t dword);

/**
 *  @brief Function that computes CRC16 (standard Modbus CRC) of a sequence of bytes
 *  @param[in]  msgByte  Sequence of bytes on which you want to calculate the CRC16
 *  @param[in]  msgByteLength  Number of bytes on which you want to calculate the CRC16
 *  @return CRC16 of a sequence of bytes
 */
uint16_t wiicom_util_CRC16(uint8_t *msgByte, uint16_t msgByteLength);

/**
 *  @brief Function that calculates the CRC16 (standard Modbus CRC) of a sequence of bytes using only one byte of that sequence and its position
 *  @param[in]  msgByte  Byte on which you want to calculate the CRC16
 *  @param[in]  msgStart Byte position within the sequence of bytes
 *  @return CRC16 calculates
 */
uint16_t wiicom_util_walkingCRC16(uint8_t msgByte, uint8_t msgStart);

/**
 *  @brief Function that stop program execution for a number of milliseconds
 *  @param[in]  number of milliseconds
 *  @return void
 */
void wiicom_util_sleep(uint32_t millisec);


#endif

