/** @file wiicom_libvcp.h
 *  @brief Wiicom virtual com port library
 *
 *  Generic library for managing a virtual com port
 *
 *  @author Marco Malano
 *  @author WIICOM s.r.l
 *  @version   1.0.0
 *  @date      2012
 *  @copyright 2012 - Wiicom - p.iva IT10710600015 - All rights reserved.
 *  @attention This library depends on @b wiicom_libutil \n
 *  Every function that expects a result, in case of an error, set the Wiicom errno and print to stderr the error occurred.
 *  @note None.
 *  @todo
 *  @deprecated
 *  @bug No bugs found.
 */

#ifndef _WIICOM_LIBVCP_H
#define _WIICOM_LIBVCP_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include "wiicom_libutil.h"

#if (defined (__WIN32__))
#include <windows.h>

typedef HANDLE t_wiicom_libvcp_handle;
#else
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>

typedef int t_wiicom_libvcp_handle;
#endif

/** @name Virtual com port Baud rate speeds
 */
#define WIICOM_VCP_BAUDRATE_50         50          /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_75         75          /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_110        110
#define WIICOM_VCP_BAUDRATE_134        134         /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_150        150         /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_200        200         /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_300        300
#define WIICOM_VCP_BAUDRATE_600        600
#define WIICOM_VCP_BAUDRATE_1200       1200
#define WIICOM_VCP_BAUDRATE_1800       1800        /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_2400       2400
#define WIICOM_VCP_BAUDRATE_4800       4800
#define WIICOM_VCP_BAUDRATE_9600       9600
#define WIICOM_VCP_BAUDRATE_19200      19200
#define WIICOM_VCP_BAUDRATE_38400      38400
#define WIICOM_VCP_BAUDRATE_57600      57600
#define WIICOM_VCP_BAUDRATE_115200     115200
#define WIICOM_VCP_BAUDRATE_128000     128000      /**< Only on Windows system */
#define WIICOM_VCP_BAUDRATE_230400     230400      /**< Only on Unix like system */
#define WIICOM_VCP_BAUDRATE_256000     256000      /**< Only on Windows system */

/** @name Data bits size
 */
#define WIICOM_VCP_DATA_BITS_SIZE_5		    5
#define WIICOM_VCP_DATA_BITS_SIZE_6		    6
#define WIICOM_VCP_DATA_BITS_SIZE_7		    7
#define WIICOM_VCP_DATA_BITS_SIZE_8		    8

/** @name Parity type
 */
#define WIICOM_VCP_NOPARITY             0   /**< No parity check */
#define WIICOM_VCP_EVENPARITY           1   /**< Use even parity check */
#define WIICOM_VCP_ODDPARITY            2   /**< Use odd parity instead of even check */


/**
 *  @brief Function that initializes the vitual com port library\n
 *  @b Warning! This function should be called before using the virtual com port.
 *  @return void
 */
void wiicom_libvcp_init();

/**
 *  @brief Function that open a VCP and returns a VCP handle
 *  @param[out]  fd  VCP handle
 *  @param[in]  device  a virtual serial port device (Ex. /dev/stty0 on Unix like, \\\\\\\\\.\\\\COM5 on Windows)
 *  @param[in]  vcp_baudrate  specifies a Baud rate speeds
 *  @param[in]  vcp_parity  specifies a parity type
 *  @param[in]  vcp_stop2bit  specifies if there is a 2 bits stop, otherwise there is 1 bit stop
 *  @param[in]  vcp_data_bits_size  it is a data bits size
 *  @return  On success, TRUE is returned, On error, FALSE is returned
 */
uint8_t wiicom_libvcp_open(t_wiicom_libvcp_handle *fd,char *device,uint32_t vcp_baudrate,uint8_t vcp_parity,uint8_t vcp_stop2bit,uint8_t vcp_data_bits_size);

/**
 *  @brief Function that close a VCP handle
 *  @param[in]  fd  VCP handle
 *  @return void
 */
void wiicom_libvcp_close(t_wiicom_libvcp_handle fd);

/**
 *  @brief Function that read data from a VCP handle
 *  @param[in]  fd  VCP handle
 *  @param[out]  buffer  buffer in which datas are loaded
 *  @param[in]  buffer_size  size of the buffer containing the data read
 *  @return On success, number of bytes received. On error, -1 is returned
 */
long wiicom_libvcp_read(t_wiicom_libvcp_handle fd, uint8_t *buffer, uint32_t buffer_size);

/**
 *  @brief Function that write data to a VCP handle
 *  @param[in]  fd  VCP handle
 *  @param[in]  buffer  it contains data to send
 *  @param[in]  buffer_size  size of the buffer to send
 *  @return On success, number of bytes transmitted. On error, -1 is returned
 */
long wiicom_libvcp_write(t_wiicom_libvcp_handle fd, uint8_t *buffer, uint32_t buffer_size);

/**
 *  @brief Function that write a string to a VCP handle
 *  @param[in]  fd  VCP handle
 *  @param[in]  string_to_write  it contains a string to send
 *  @return On success, number of bytes transmitted. On error, -1 is returned
 */
long wiicom_libvcp_write_string(t_wiicom_libvcp_handle fd,char *string_to_write);

/**
 *  @brief Function that de-initialize the vitual com port library\n
 *  @b Warning! This function is called when not used any more a VCP or, in general, before the close of the program.
 *  @return void
 */
void wiicom_libvcp_deinit();

/**
 *  @brief Function that get all system VCP\n
 *  @b Warning! This function works only on windows and linux system
 *  @param[out]  ports  array width discoverd ports
 *  @return number of discoverd ports
 */
int wiicom_libvcp_getcomports(char ports[][15]);

#endif
